import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';

class AddFollowUpExample extends StatefulWidget {
  AddFollowUpExample({
    @required this.userNumbers,
    @required this.prospectNumbers,
  });

  final List<PhoneNumber> userNumbers, prospectNumbers;
  @override
  _AddFollowUpExampleState createState() => _AddFollowUpExampleState();
}

class _AddFollowUpExampleState extends State<AddFollowUpExample> {
  String _title = "";
  String _description = "";
  List<PhoneNumber> _prospectNumbers = [];
  List<PhoneNumber> _userNumbers = [];
  PhoneNumber _prospectNumber;
  PhoneNumber _userNumber;
  int _type = 0;
  int _offset = 0;
  bool _dismiss = true;
  DateTime _date = DateTime.now();
  bool _sendEmail = false;
  bool _sendSMS = false;

  @override
  void initState() {
    _userNumbers..addAll(widget?.userNumbers ?? []);
    _prospectNumbers..addAll(widget?.prospectNumbers ?? []);
    setState(() {
      if (_userNumbers != null && _userNumbers.isNotEmpty)
        _userNumber = _userNumbers?.first;
      if (_prospectNumbers != null && _prospectNumbers.isNotEmpty)
        _prospectNumber = _prospectNumbers?.first;
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Add Follow Up"),
      ),
      body: SingleChildScrollView(
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            ListTile(
              title: DropdownButtonFormField(
                items: [
                  DropdownMenuItem<int>(
                    value: 0,
                    child: Text("Task"),
                  ),
                  DropdownMenuItem<int>(
                    value: 1,
                    child: Text("Appointment"),
                  ),
                  DropdownMenuItem<int>(
                    value: 2,
                    child: Text("Scheduled Call"),
                  ),
                ],
                value: _type,
                onChanged: (int value) {
                  setState(() {
                    _type = value;
                  });
                },
              ),
            ),
            ListTile(
              title: TextField(
                onChanged: (String value) {
                  setState(() {
                    _title = value;
                  });
                },
                decoration: InputDecoration(hintText: "Title"),
              ),
            ),
            ListTile(
              title: TextField(
                onChanged: (String value) {
                  setState(() {
                    _description = value;
                  });
                },
                decoration: InputDecoration(hintText: "Description"),
              ),
            ),
            ListTile(
              title: Text("Dismiss On Change"),
              trailing: Switch.adaptive(
                value: _dismiss,
                onChanged: (bool value) {
                  setState(() {
                    _dismiss = value;
                  });
                },
              ),
            ),
            ListTile(
              title: Text("Text Message"),
              trailing: Switch.adaptive(
                value: _sendSMS,
                onChanged: (bool value) {
                  setState(() {
                    _sendSMS = value;
                  });
                },
              ),
            ),
            ListTile(
              title: Text("Email"),
              trailing: Switch.adaptive(
                value: _sendEmail,
                onChanged: (bool value) {
                  setState(() {
                    _sendEmail = value;
                  });
                },
              ),
            ),
            ListTile(
              title: Text("Alert Date"),
              subtitle:
                  _date == null ? Text("Pick Date") : Text(_date.toString()),
              onTap: () {
                showDatePicker(
                  context: context,
                  initialDate: _date,
                  firstDate: DateTime.now(),
                  lastDate: null,
                ).then((value) {
                  if (value != null) {
                    setState(() {
                      _date = value;
                    });
                  }
                });
              },
            ),
            ListTile(
              title: DropdownButtonFormField(
                items: _userNumbers
                    .map(
                      (p) => DropdownMenuItem<PhoneNumber>(
                            value: p,
                            child: Text(p.toString()),
                          ),
                    )
                    .toList(),
                value: _userNumber,
                onChanged: (PhoneNumber value) {
                  setState(() {
                    _userNumber = value;
                  });
                },
              ),
            ),
            ListTile(
              title: DropdownButtonFormField(
                items: _prospectNumbers
                    .map(
                      (p) => DropdownMenuItem<PhoneNumber>(
                            value: p,
                            child: Text(p.toString()),
                          ),
                    )
                    .toList(),
                value: _prospectNumber,
                onChanged: (PhoneNumber value) {
                  setState(() {
                    _prospectNumber = value;
                  });
                },
              ),
            ),
            RaisedButton.icon(
              icon: Icon(Icons.save),
              label: Text("Save"),
              onPressed: () {
                String _alertType = "";
                if (_sendEmail && _sendSMS) {
                  _alertType = "Email, TextMessage";
                } else if (_sendEmail && !_sendSMS) {
                  _alertType = "Email";
                } else if (!_sendEmail && _sendSMS) {
                  _alertType = "TextMessage";
                }
                return Navigator.pop(context, {
                  "type": _type,
                  "title": _title ?? "",
                  "description": _description ?? "",
                  "date_time": _date ?? DateTime.now(),
                  "dismiss_on_change": _dismiss ?? true,
                  "alert_type": _alertType ?? "",
                  "alert_offset": _offset ?? 0,
                  "prospect_number": _prospectNumber?.toString(),
                  "user_number": _userNumber?.toString()
                });
              },
            )
          ],
        ),
      ),
    );
  }
}
