import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';
import 'package:unify_api_example/common/index.dart';
import 'package:unify_api_example/utils/index.dart';

class ContactsExample extends StatefulWidget {
  final UnifyClient client;

  const ContactsExample({Key key, this.client}) : super(key: key);
  @override
  _ContactsExampleState createState() => _ContactsExampleState();
}

class _ContactsExampleState extends State<ContactsExample> {
  List<Contact> _items;

  @override
  void initState() {
    widget.client.listContacts().then((items) {
      setState(() {
        _items = items;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Contacts"),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.add),
            onPressed: () {
              Navigator.push<ContactDetails>(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) {
                      return new AddContactExample(
                        client: widget.client,
                      );
                    },
                    fullscreenDialog: true,
                  )).then((value) {
                if (value != null) {
                  ContactDetails _info = value;
                  widget.client.addContact(
                    firstName: _info?.firstName,
                    lastName: _info?.lastName,
                    middleName: _info?.middleName,
                    suffix: _info?.suffix,
                    secondaryFirstName: _info?.secondaryFirstName,
                    secondaryMiddleName: _info?.secondaryMiddleName,
                    secondaryLastName: _info?.secondaryLastName,
                    currentAddress: _info?.currentAddress,
                    cellPhone: _info?.cellPhone,
                    homePhone: _info?.homePhone,
                    officePhone: _info?.officePhone,
                    faxPhone: _info?.faxPhone,
                    secondaryCellPhone: _info?.secondaryCellPhone,
                    email: _info?.email,
                    secondaryEmail: _info?.secondaryEmail,
                    loanNumber: _info?.loanNumber,
                    contactPriority: _info?.priority,
                    companyCategory: _info?.companyCategory,
                    groups: _info?.groups,
                  );
                }
              });
            },
          ),
        ],
      ),
      body: Builder(
        builder: (BuildContext context) {
          if (_items == null) return Center(child: CircularProgressIndicator());
          if (_items.isEmpty) return Center(child: Text("No Contacts Found"));
          return ListView.builder(
            itemBuilder: (BuildContext context, int index) {
              final i = _items[index];
              return ListTile(
                title: Text(i.displayName),
                subtitle: Text(i.lastActivity),
                trailing: JsonButton(json: i?.json),
                onTap: () {
                  Navigator.push(
                    context,
                    MaterialPageRoute(
                      builder: (BuildContext context) =>
                          ContactDetailsExample(contact: i.getDetails()),
                    ),
                  );
                },
              );
            },
            itemCount: _items?.length ?? 0,
          );
        },
      ),
    );
  }
}

class ContactDetailsExample extends StatefulWidget {
  final Future<ContactDetails> contact;

  const ContactDetailsExample({Key key, @required this.contact})
      : super(key: key);
  @override
  _ContactDetailsExampleState createState() => _ContactDetailsExampleState();
}

class _ContactDetailsExampleState extends State<ContactDetailsExample> {
  ContactDetails _details;

  @override
  void initState() {
    widget.contact.then((details) {
      setState(() {
        _details = details;
      });
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Contact Details"),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.delete),
            onPressed: () {
              _details.delete().then((_) {
                Navigator.pop(context);
              }).catchError((error) {
                print("Error: $error");
              });
            },
          ),
          IconButton(
            icon: Icon(Icons.edit),
            onPressed: () {
              Navigator.push<ContactDetails>(
                  context,
                  MaterialPageRoute(
                    builder: (BuildContext context) {
                      return new AddContactExample(
                        client: _details.client,
                        contact: _details,
                      );
                    },
                    fullscreenDialog: true,
                  )).then((value) {
                if (value != null) {
                  ContactDetails _info = value;
                  _details.edit(
                    firstName: _info?.firstName,
                    lastName: _info?.lastName,
                    middleName: _info?.middleName,
                    suffix: _info?.suffix,
                    secondaryFirstName: _info?.secondaryFirstName,
                    secondaryMiddleName: _info?.secondaryMiddleName,
                    secondaryLastName: _info?.secondaryLastName,
                    currentAddress: _info?.currentAddress,
                    cellPhone: _info?.cellPhone,
                    homePhone: _info?.homePhone,
                    officePhone: _info?.officePhone,
                    faxPhone: _info?.faxPhone,
                    secondaryCellPhone: _info?.secondaryCellPhone,
                    email: _info?.email,
                    secondaryEmail: _info?.secondaryEmail,
                    loanNumber: _info?.loanNumber,
                    contactPriority: _info?.priority,
                    companyCategory: _info?.companyCategory,
                    groups: _info?.groups,
                  );
                }
              });
            },
          ),
          JsonButton(json: _details?.json),
        ],
      ),
      body: FutureBuilder<ContactDetails>(
        initialData: _details,
        future: widget.contact,
        builder:
            (BuildContext context, AsyncSnapshot<ContactDetails> snapshot) {
          if (snapshot?.data == null)
            return Center(child: CircularProgressIndicator());

          ContactDetails details = snapshot.data;
          return ListView(
            children: <Widget>[
              ListTile(
                title: Text("Full Name"),
                subtitle: Text(details.fullName),
              ),
              ListTile(
                title: Text("Email"),
                subtitle: Text(details.email),
              ),
              ListTile(
                title: Text("Cell Phone"),
                subtitle: Text(details.cellPhone.toString()),
              ),
              ListTile(
                title: Text("Home Phone"),
                subtitle: Text(details.homePhone.toString()),
              ),
              ListTile(
                title: Text("Office Phone"),
                subtitle: Text(details.officePhone.toString()),
              ),
              ListTile(
                title: Text("Notes"),
                subtitle: Text("Tap To View"),
                trailing: IconButton(
                  icon: Icon(Icons.note_add),
                  onPressed: () {
                    Navigator.push<String>(
                        context,
                        MaterialPageRoute(
                          builder: (BuildContext context) {
                            return new AddNoteExample();
                          },
                          fullscreenDialog: true,
                        )).then((value) {
                      if (value != null) {
                        details.addNote(value);
                      }
                    });
                  },
                ),
                onTap: () {
                  var items = details?.notes ?? [];
                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.note),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
              ListTile(
                title: Text("Tracker"),
                subtitle: Text("Tap To View"),
                trailing: IconButton(
                    icon: Icon(Icons.timer),
                    onPressed: () {
                      Navigator.push<Map<String, dynamic>>(
                          context,
                          MaterialPageRoute(
                            builder: (BuildContext context) {
                              return new AddLogResponseExample(
                                client: details.client,
                                responses: details.listResponses(),
                              );
                            },
                            fullscreenDialog: true,
                          )).then((value) {
                        if (value != null) {
                          details.addLogResponse(
                            action: value["action"],
                            comment: value["comment"],
                          );
                        }
                      });
                    }),
                onTap: () {
                  var items = details?.tracker ?? [];

                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.type),
                            subtitle: Text(i.description),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
              ListTile(
                title: Text("Tasks"),
                subtitle: Text("Tap To View"),
                trailing: IconButton(
                    icon: Icon(Icons.event),
                    onPressed: () async {
                      var _info = await details.client.getInfo();
                      Navigator.push<Map<String, dynamic>>(
                          context,
                          MaterialPageRoute(
                            builder: (BuildContext context) {
                              return new AddFollowUpExample(
                                userNumbers: _info.numbers,
                                prospectNumbers: details.numbers,
                              );
                            },
                            fullscreenDialog: true,
                          )).then((value) {
                        if (value != null) {
                          details.addFollowUp(
                            type: value["type"] ?? 0,
                            title: value["title"] ?? "",
                            description: value["description"] ?? "",
                            date: value["date_time"] ?? DateTime.now(),
                            dismissOnChange: value["dismiss_on_change"] ?? true,
                            alertType: value["alert_type"] ?? "",
                            alertOffset: value["alert_offset"] ?? 0,
                            prospectNumber: value["prospect_number"],
                            userNumber: value["user_number"],
                          );
                        }
                      });
                    }),
                onTap: () {
                  var items = details?.tasks ?? [];
                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i.type),
                            subtitle: Text(i?.dateTime?.toString() ?? ""),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
              ListTile(
                title: Text("Loan History"),
                subtitle: Text("Tap To View"),
                onTap: () {
                  var items = details?.loans ?? [];
                  List<Widget> _children = [];
                  _children = items
                      .map((i) => ListTile(
                            title: Text(i?.loanNumber ?? ""),
                            subtitle: Text(i?.id ?? ""),
                            trailing: JsonButton(json: i?.json),
                          ))
                      .toList();
                  showList(context, _children);
                },
              ),
            ],
          );
        },
      ),
    );
  }
}
