import 'dart:async';

import 'package:bloc/bloc.dart';
import 'package:flutter/cupertino.dart';
import 'package:get_it/get_it.dart';
import 'package:rxdart/rxdart.dart';

import '../../services/database/database.dart';
import 'bloc.dart';

class SearchBloc extends Bloc<SearchEvent, SearchState> {
  @override
  SearchState get initialState => SearchStateEmpty();

  @override
  Stream<SearchState> mapEventToState(
    SearchEvent event,
  ) async* {
    if (event is TextChanged) {
      final searchTerm = event.text;
      if (searchTerm.isEmpty) {
        yield SearchStateEmpty();
      } else {
        yield SearchStateLoading();
        try {
          final _results = <SearchResultItem>[];
          for (var type in event.types) {
            switch (type) {
              case SearchResultType.loans:
                final _items = await GetIt.instance<DatabaseService>()
                    .loansDao
                    .searchLoans(event.user.id, searchTerm);
                if (_items != null) {
                  _results.addAll(_items
                      .map((i) => SearchResultItem(
                            type: type,
                            data: i.toJson(),
                          ))
                      .toList());
                }
                break;
              case SearchResultType.contacts:
                final _items = await GetIt.instance<DatabaseService>()
                    .contactsDao
                    .searchContacts(event.user.id, searchTerm);
                if (_items != null) {
                  _results.addAll(_items
                      .map(
                          (i) => SearchResultItem(data: i.toJson(), type: type))
                      .toList());
                }
                break;
              case SearchResultType.leads:
                final _items = await GetIt.instance<DatabaseService>()
                    .leadsDao
                    .searchLeads(event.user.id, searchTerm);
                if (_items != null) {
                  _results.addAll(_items
                      .map(
                          (i) => SearchResultItem(data: i.toJson(), type: type))
                      .toList());
                }
                break;
              case SearchResultType.alerts:
                final _items = await GetIt.instance<DatabaseService>()
                    .dashboardDao
                    .searchAlerts(event.user.id, searchTerm);
                if (_items != null) {
                  _results.addAll(_items
                      .map(
                          (i) => SearchResultItem(data: i.toJson(), type: type))
                      .toList());
                }
                break;
              case SearchResultType.video:
                final _items = await GetIt.instance<DatabaseService>()
                    .videoDao
                    .searchVideos(event.user.id, searchTerm);
                if (_items != null) {
                  _results.addAll(_items
                      .map(
                          (i) => SearchResultItem(data: i.toJson(), type: type))
                      .toList());
                }
                break;
              default:
            }
          }
          yield SearchStateSuccess(_results);
        } catch (error) {
          yield SearchStateError('something went wrong => $error');
        }
      }
    }
  }

  @override
  Stream<SearchState> transformEvents(Stream<SearchEvent> events,
      Stream<SearchState> Function(SearchEvent event) next) {
    Observable<SearchEvent> _events = events;
    return super.transformEvents(
      _events.debounceTime(Duration(milliseconds: 200)),
      next,
    );
  }
}

enum SearchResultType { contacts, loans, leads, alerts, video }

class SearchResultItem {
  const SearchResultItem({
    @required this.type,
    @required this.data,
  });

  final Map<String, dynamic> data;
  final SearchResultType type;
}
