import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;


import '../database.dart';
import '../tables/contact_group.dart';

part 'contact_group.g.dart';

@UseDao(
  tables: [ContactGroupRecords],
  queries: {
    'groupsByUser':
        'SELECT * FROM contactGroup WHERE user_id = :userId ORDER BY last_updated ASC'
  }
)
class ContactGroupDao extends DatabaseAccessor<DatabaseService> with _$ContactGroupDaoMixin {
  ContactGroupDao(DatabaseService db) : super(db);

  Future batchUpdate(String userId, List<api.ContactGroup> groups) async {
    final _items = <ContactGroupRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var group in groups){
      final _item = ContactGroupRecordsCompanion(
        userId: addField<String>(userId),
        groupName: addField<String>(group.name),
        groupId: addField<String>(group.id),
        count: addField<int>(group.count),
        lastUpdated: addField<DateTime>(_updated)
      );
      _items.add(_item);
    }
    await into(contactGroupRecords).insertAll(_items, orReplace: true);
    await cleanGroups(userId, _updated);
  }

  Future addItem(String groupName, String userId, int count) async {
    final _item = ContactGroupRecordsCompanion(
      groupName: addField<String>(groupName),
      userId: addField<String>(userId),
      count: addField<int>(count),
      lastUpdated: addField<DateTime>(DateTime.now())
    );
    return await transaction(() async {
      await into(contactGroupRecords).insert(_item, orReplace: true);
    });
  }

  Future cleanGroups(String userId, DateTime lastUpdated) async {
    return await (delete(contactGroupRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }

  Future removeItem(String groupId, String userId) async {
    return await transaction(() async {
      return (delete(contactGroupRecords)
            ..where((l) => l.userId.equals(userId))
            ..where((l) => l.groupId.equals(groupId)))
          .go();
    });
  }
}