import 'package:moor/moor.dart';

import '../database.dart';
import '../tables/home.dart';

part 'home_dashboard.g.dart';

@UseDao(
  tables: [HomeRecords],
  queries: {
    'itemById':
        'SELECT * FROM home WHERE user_id = :userId AND item_id = :itemId LIMIT 1',
    'itemsForUser':
        'SELECT * FROM home WHERE user_id = :userId ORDER BY order_index ASC',
  },
)
class HomeDao extends DatabaseAccessor<DatabaseService> with _$HomeDaoMixin {
  HomeDao(DatabaseService db) : super(db);

  Future addItem(String tag, String userId) async {
    final _item = HomeRecordsCompanion(
      userId: addField<String>(userId),
      tag: addField<String>(tag),
    );
    return await transaction(() async {
      await into(homeRecords).insert(
        _item,
        orReplace: true,
      );
    });
  }

  Future removeItem(int itemId, String userId) async {
    return await transaction(() async {
      return (delete(homeRecords)
            ..where((l) => l.userId.equals(userId))
            ..where((l) => l.itemId.equals(itemId)))
          .go();
    });
  }

  Future updateItems(String tag, int itemId, int orderIndex,
      {@required String userId}) async {
    final now = DateTime.now();
    final _widget = await itemById(userId, itemId);
    if (_widget != null && _widget.isNotEmpty) {
      return await (update(homeRecords)
            ..where((t) => t.userId.equals(userId))
            ..where((t) => t.itemId.equals(itemId)))
          .write(_widget.first
              .copyWith(tag: tag, orderIndex: orderIndex, lastUpdated: now));
    }
    return await into(homeRecords).insert(
      HomeRecordsCompanion(
          userId: addField<String>(userId),
          tag: addField<String>(tag),
          itemId: addField<int>(itemId),
          orderIndex: addField<int>(orderIndex)),
      orReplace: true,
    );
  }
}
