import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_email_sender/flutter_email_sender.dart';
import 'package:flutter_sms/flutter_sms.dart';
import 'package:permission/permission.dart';
import 'package:url_launcher/url_launcher.dart';

sendSMS(String message, {List<String> recipients = const <String>[]}) async {
  if (Platform.isAndroid) {
    var permissionNames =
        await Permission.requestPermissions([PermissionName.SMS]);
  }
  String _result =
      await FlutterSms.sendSMS(message: message, recipients: recipients)
          .catchError((onError) {
    print(onError);
  });
  print(_result);
}

makePhoneCall(BuildContext context, String phone) async {
  if (phone.length > 0) {
    String call = phone.replaceAll(RegExp(r'-'), '');
    String url = 'tel:+1$call';
    launchURL(url);
  } else {
    showAlertPopup(context, 'Info', 'Phone Number Not Found');
  }
}

sendEmail(BuildContext context,
    {List<String> recipients = const <String>[],
    String body = "",
    String subject = "",
    List<String> cc = const <String>[],
    List<String> bcc = const <String>[]}) async {
  final Email _email = Email(
    body: body,
    subject: subject,
    recipients: recipients,
    cc: cc,
    bcc: bcc,
    // attachmentPath: '/path/to/attachment.zip',
  );
  try {
    await FlutterEmailSender.send(_email);
  } catch (e) {
    if (recipients.isNotEmpty) {
      String url = 'mailto:${recipients.first.toString()}';
      launchURL(url);
    } else {
      showAlertPopup(context, 'Info', 'Email Not Found');
    }
  }
}


launchURL(String url,
    {bool forceSafariVC = false, bool forceWebView = false}) async {
  try {
    if (await canLaunch(url)) {
      await launch(url, forceSafariVC: forceSafariVC, forceWebView: false);
    }
  } catch (e) {
    print(e);
  }
}

void showAlertPopup(BuildContext context, String title, String detail) async {
  void showDemoDialog<T>({BuildContext context, Widget child}) {
    showDialog<T>(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) => child,
    );
  }

  return showDemoDialog<Null>(
      context: context,
      child: AlertDialog(
        title: Text(title),
        content: Text(detail),
        actions: <Widget>[
          FlatButton(
              child: Text('OK'),
              onPressed: () {
                Navigator.of(context).pop();
              }),
        ],
      ));
}