import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/alerts/bloc.dart';
import '../../../data/blocs/auth/auth_bloc.dart';
import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import 'tile.dart';

class AlertFolderScreen extends StatelessWidget {
  const AlertFolderScreen({Key key, @required this.folder}) : super(key: key);

  final DashboardAlertType folder;

  @override
  Widget build(BuildContext context) {
    final alert = BlocProvider.of<AlertsBloc>(context);
    final _user = AuthBloc.currentUser(context);
    return StreamBuilder(
        stream: alert.state,
        builder: (context, snapshot) {
          final _state = snapshot.data;
          return Column(
            children: <Widget>[
              if (_state is AlertsLoadingState && _state.folder == folder)
                LinearProgressIndicator(),
              Expanded(
                child: Container(
                  child: StreamBuilder<List<Dashboard>>(
                    stream: GetIt.instance<DatabaseService>()
                        .dashboardDao
                        .watchAlertsByFolder(
                            _user.userID, describeEnum(folder)),
                    builder: (context, snapshot) {
                      if (snapshot.hasData) {
                        final _alerts = snapshot.data;
                        if (_alerts.isEmpty) {
                          if (_state is AlertsLoadingState) {
                            return Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              crossAxisAlignment: CrossAxisAlignment.center,
                              children: <Widget>[
                                Text('No Alerts Found'),
                                SizedBox(
                                  height: 20,
                                ),
                                CircularProgressIndicator()
                              ],
                            );
                          }
                          return Center(child: Text('No Alerts Found'));
                        }
                        return Scrollbar(
                          child: RefreshIndicator(
                            child: ListView.separated(
                                separatorBuilder: (context, index) =>
                                    Divider(height: 1),
                                itemCount: _alerts.length,
                                itemBuilder: (context, index) {
                                  final _alert = _alerts[index];
                                  // print('${_alert.folder}');
                                  return AlertRowTile(
                                      alert: _alert, type: ViewType.detailed);
                                }),
                            onRefresh: () async {
                              refresh(context, folder);
                              await Future.delayed(Duration(seconds: 3));
                            },
                          ),
                        );
                      }
                      return Center(child: CircularProgressIndicator());
                    },
                  ),
                ),
              )
            ],
          );
        });
  }

  static refresh(BuildContext context, DashboardAlertType folder) {
    final alerts = BlocProvider.of<AlertsBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    return alerts.dispatch(RefreshAlerts(
      info: _user,
      folder: folder,
      scaffold: scaffold,
      context: context
    ));
  }
}
