import 'package:after_layout/after_layout.dart';
import 'package:flutter/material.dart';
import 'package:table_calendar/table_calendar.dart';
import 'package:unify_api/unify.dart';

import '../../../data/blocs/auth/auth_bloc.dart';
import '../../../plugins/calendar/classes/calender.dart';

class UnifyCalendar extends StatefulWidget {
  const UnifyCalendar({
    @required this.calendarController,
    @required this.start,
    @required this.end,
    @required this.client,
    this.onDaySelected,
    this.onVisibleDaysChanged,
    Key key,
  }) : super(key: key);

  // final Map<DateTime, List<dynamic>> events;
  final CalendarController calendarController;
  final DateTime start, end;
  final UnifyClient client;
  final Function(DateTime, List<dynamic>) onDaySelected;

  final Function(DateTime, DateTime, CalendarFormat) onVisibleDaysChanged;

  @override
  _UnifyCalendarState createState() => _UnifyCalendarState();

  static final Map<DateTime, List> holidays = {
    DateTime(2019, 1, 1): ['New Year\'s Day'],
    DateTime(2019, 1, 6): ['Epiphany'],
    DateTime(2019, 2, 14): ['Valentine\'s Day'],
    DateTime(2019, 4, 21): ['Easter Sunday'],
    DateTime(2019, 4, 22): ['Easter Monday'],
  };
}

class _UnifyCalendarState extends State<UnifyCalendar>
    with AfterLayoutMixin<UnifyCalendar> {
  Map<DateTime, List<dynamic>> events = <DateTime, List<dynamic>>{};

  @override
  void didUpdateWidget(UnifyCalendar oldWidget) {
    if (oldWidget.calendarController.selectedDay !=
        widget.calendarController.selectedDay) {
      _loadEventsForCal(
        widget.client,
        widget.calendarController?.selectedDay ?? DateTime.now(),
      );
    }
    super.didUpdateWidget(oldWidget);
  }

  void _loadEventsForCal(UnifyClient client, DateTime selected) async {
    // final _events = await CalendarUtils.getEventsForCalendar(
    //   id,
    //   startDate: selected.subtract(Duration(days: 30)),
    //   endDate: selected.add(Duration(days: 30)),
    // );

    final _events = await client.listTasksAll(date: selected);

    for (var event in _events) {
      if (events[event.dateTime] == null) {
        events[event.dateTime] = [];
      }
      if (mounted)
        setState(() {
          events[event.dateTime].add(event.title);
        });
    }
  }

  @override
  Widget build(BuildContext context) {
    return TableCalendar(
      calendarController: widget.calendarController,
      events: events,
      holidays: UnifyCalendar.holidays,
      startingDayOfWeek: StartingDayOfWeek.monday,
      calendarStyle: CalendarStyle(
        selectedColor: Colors.deepOrange[400],
        todayColor: Colors.deepOrange[200],
        markersColor: Colors.brown[700],
        outsideDaysVisible: false,
      ),
      headerStyle: HeaderStyle(
        formatButtonTextStyle:
            TextStyle().copyWith(color: Colors.white, fontSize: 15.0),
        formatButtonDecoration: BoxDecoration(
          color: Colors.deepOrange[400],
          borderRadius: BorderRadius.circular(16.0),
        ),
      ),
      onDaySelected: (day, items) async {
        widget.onDaySelected(day, null);
        final client = AuthBloc.currentInfo(context).client;
        final _events = await client.listTasksAll(date: day);
        for (var event in _events) {
          if (events[event.dateTime] == null) {
            events[event.dateTime] = [];
          }
          if (mounted)
            setState(() {
              events[event.dateTime].add(event.title);
            });
        }
        print('Events: ${_events.length}');
        widget.onDaySelected(day, _events);
      },
      onVisibleDaysChanged: widget.onVisibleDaysChanged,
    );
  }

  @override
  void afterFirstLayout(BuildContext context) {
    _loadEventsForCal(
      widget.client,
      widget.calendarController?.selectedDay ?? DateTime.now(),
    );
  }
}
