import 'package:flutter/material.dart';

class AppInputField extends StatelessWidget {
  final Widget leading, trailing, subtitle;
  final TextEditingController controller;
  final String name, onError;
  final bool autoFocus, required, multiLine, numeric;
  final ValueChanged<String> onChanged;
  final FocusNode focus;

  AppInputField({
    this.controller,
    this.name = "Item",
    this.autoFocus = false,
    this.required,
    this.leading,
    this.subtitle,
    this.trailing,
    this.multiLine = false,
    this.numeric = false,
    this.onError,
    this.onChanged,
    this.focus,
  });

  @override
  Widget build(BuildContext context) {
    return ListTile(
      leading: leading,
      title: TextFormField(
        autofocus: autoFocus,
        focusNode: focus,
        controller: controller,
        decoration: InputDecoration(labelText: name),
        keyboardType: multiLine
            ? TextInputType.multiline
            : getKeyboard(name, numeric: numeric),
        validator: (val) => validator(val,
            type: name, required: required, onError: onError, numeric: numeric),
        onFieldSubmitted: onChanged,
        onSaved: onChanged,
        maxLines: multiLine ? null : 1,
      ),
      subtitle: subtitle,
      trailing: trailing,
    );
  }
}

String validator(String value,
    {String type,
    bool required = false,
    String onError,
    bool numeric = false}) {
  if (numeric) {
    if (value.isEmpty && required) return onError ?? 'Number is required.';
    final RegExp nameExp = RegExp(r'^[A-Za-z ]+$');
    if (nameExp.hasMatch(value))
      return 'Please enter only numerial characters.';
    num _number = num.tryParse(value);
    if (_number == null) return 'Please enter a valid number';
  }
  if (required ?? false) {
    if (value != null && value.isEmpty) return onError ?? '$type Required';
  }

  return null;
}

TextInputType getKeyboard(String type, {bool numeric = false}) {
  if (numeric) return TextInputType.number;
  if (type.contains("Email")) return TextInputType.emailAddress;
  return TextInputType.text;
}