import 'package:contact_picker/contact_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart' as unify;

import '../../data/classes/phone.dart';
import '../../data/utils/is_empty.dart';
import '../../data/utils/phone_email_actions.dart';

class PhoneTile extends StatelessWidget {
  final String label;
  final IconData icon;
  final unify.PhoneNumber number;
  final VoidCallback callPressed, smsPressed;


  PhoneTile({
    this.label,
    this.number,
    this.icon,
    this.callPressed,
    this.smsPressed,
  });

  @override
  Widget build(BuildContext context) {
    // var _raw = (number ?? "")
    //     .replaceAll("(", "")
    //     .replaceAll(")", "")
    //     .replaceAll("-", "")
    //     .replaceAll(" ", "")
    //     .trim();
    var _raw = number?.raw();
    if (isNullOrEmpty(_raw)) {
      return ListTile(
        leading: Icon(icon ?? Icons.phone),
        title: Text(
          isNullOrEmpty(label) ? 'Phone Number' : label,
          textScaleFactor: 1,
        ),
        subtitle: const Text(
          "No Number Found",
          textScaleFactor: 1,
        ),
      );
    }

    return ListTile(
      leading: Icon(icon ?? Icons.phone),
      title: Text(
        label ?? 'Phone Number',
        textScaleFactor: 1,
      ),
      subtitle: Text(
        number?.toString(),
        textScaleFactor: 1,
      ),
      trailing: IconButton(
        icon: Icon(Icons.message),
        onPressed: () {
          if (smsPressed != null) smsPressed();
          return sendSMS("", recipients: [_raw]);
        },
      ),
      onTap: () {
        if (callPressed != null) callPressed();
        return makePhoneCall(context, _raw);
      },
    );
  }
}

class PhoneInputTile extends StatefulWidget {
  final String label;
  final Phone number;
  final ValueChanged<Phone> numberChanged;
  final bool showExt;

  PhoneInputTile({
    this.numberChanged,
    this.number,
    this.label,
    this.showExt = false,
  });

  @override
  _PhoneInputTileState createState() => _PhoneInputTileState();
}

class _PhoneInputTileState extends State<PhoneInputTile> {
  TextEditingController _areaCode, _prefix, _number, _ext;
  final _formKey = GlobalKey<FormState>();
  bool _isEditing = false;
  @override
  void initState() {
    _areaCode = TextEditingController(text: widget?.number?.areaCode ?? "");
    _prefix = TextEditingController(text: widget?.number?.prefix ?? "");
    _number = TextEditingController(text: widget?.number?.number ?? "");
    _ext = TextEditingController(text: widget?.number?.ext ?? "");
    super.initState();
  }

  Phone get phone {
    var _phone = Phone(
      label: widget?.label?.toLowerCase()?.replaceAll("number", "")?.trim() ??
          "number",
      areaCode: _areaCode?.text ?? "",
      prefix: _prefix?.text ?? "",
      number: _number?.text ?? "",
    );
    // if (_phone.raw().isEmpty) return null;
    return _phone;
  }

  void _clear() {
    _areaCode.clear();
    _prefix.clear();
    _number.clear();
    _ext.clear();
  }

  @override
  Widget build(BuildContext context) {
    final Widget searchContacts = IconButton(
        icon: Icon(Icons.search),
        onPressed: () async {
          final _contactPicker = new ContactPicker();
          final contact = await _contactPicker.selectContact();
          if (contact != null) {
            var _value = contact?.phoneNumber?.number;
            var _newPhone = Phone.fromString(_value);
            _clear();
            setState(() {
              _areaCode.text = _newPhone?.areaCode;
              _prefix.text = _newPhone?.prefix;
              _number.text = _newPhone?.number;
              _ext.text = _newPhone?.ext;
            });
          }
        });
    if (!_isEditing)
      return ListTile(
        leading: searchContacts,
        title: Text(
          widget?.label ?? "Phone Number",
          style: Theme.of(context).textTheme?.body1,
        ),
        subtitle: phone == null || phone.raw().isEmpty
            ? Text("No Number Added")
            : Text(phone.toString()),
        trailing:
            Icon(phone == null || phone.raw().isEmpty ? Icons.add : Icons.edit),
        onTap: () {
          setState(() {
            _isEditing = !_isEditing;
          });
        },
      );
    return ListTile(
      leading: searchContacts,
      title: Form(
        autovalidate: true,
        key: _formKey,
        child: Row(
          children: <Widget>[
            SizedBox(
                width: 75.0,
                child: Container(
                  padding: EdgeInsets.symmetric(horizontal: 5.0),
                  child: TextFormField(
                    decoration: InputDecoration(labelText: "Area"),
                    controller: _areaCode,
                    maxLength: 3,
                    keyboardType: TextInputType.number,
                  ),
                )),
            SizedBox(
                width: 75.0,
                child: Container(
                  padding: EdgeInsets.symmetric(horizontal: 5.0),
                  child: TextFormField(
                    decoration: InputDecoration(labelText: "Prefix"),
                    controller: _prefix,
                    maxLength: 3,
                    keyboardType: TextInputType.number,
                  ),
                )),
            Expanded(
                child: Container(
              padding: EdgeInsets.symmetric(horizontal: 5.0),
              child: TextFormField(
                decoration: InputDecoration(labelText: "Number"),
                controller: _number,
                maxLength: 4,
                keyboardType: TextInputType.number,
              ),
            )),
            widget.showExt
                ? SizedBox(
                    width: 75.0,
                    child: Container(
                      padding: EdgeInsets.symmetric(horizontal: 5.0),
                      child: TextFormField(
                        decoration: InputDecoration(labelText: "Ext."),
                        controller: _ext,
                        maxLength: 15,
                        keyboardType: TextInputType.number,
                      ),
                    ))
                : Container(),
          ],
        ),
        onChanged: () {
          widget.numberChanged(phone);
        },
      ),
      trailing: Icon(Icons.close),
      onTap: () {
        setState(() {
          _isEditing = !_isEditing;
        });
      },
    );
  }
}

class PhonePickerTile extends StatelessWidget {
  final List<Phone> phones;
  final Phone initialItem;
  final ValueChanged<int> phoneChanged;
  final Widget title, leading, trailing, subtitle;

  PhonePickerTile({
    @required this.phones,
    @required this.title,
    this.initialItem,
    this.phoneChanged,
    this.leading,
    this.subtitle,
    this.trailing,
  });

  @override
  Widget build(BuildContext context) {
    return ListTile(
      leading: leading,
      title: title,
      subtitle: subtitle,
      onTap: () {
        List<Phone> _phones = phones;
        final FixedExtentScrollController scrollController =
            FixedExtentScrollController(
                initialItem: _phones.indexOf(initialItem ?? _phones.first));
        showCupertinoModalPopup<void>(
          context: context,
          builder: (BuildContext context) {
            return _buildBottomPicker(
              CupertinoPicker(
                scrollController: scrollController,
                itemExtent: 32,
                backgroundColor: CupertinoColors.white,
                onSelectedItemChanged: phoneChanged,
                children: List<Widget>.generate(_phones.length, (int index) {
                  return Center(
                    child: Text(_phones[index].toString()),
                  );
                }),
              ),
            );
          },
        );
      },
    );
  }

  Widget _buildBottomPicker(Widget picker) {
    return Container(
      height: 216,
      padding: const EdgeInsets.only(top: 6.0),
      color: CupertinoColors.white,
      child: DefaultTextStyle(
        style: const TextStyle(
          color: CupertinoColors.black,
          fontSize: 22.0,
        ),
        child: GestureDetector(
          // Blocks taps from propagating to the modal sheet and popping.
          onTap: () {},
          child: SafeArea(
            top: false,
            child: picker,
          ),
        ),
      ),
    );
  }
}
