import 'package:fab_menu/fab_menu.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:unify_api/unify.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/utils/navigate.dart';
import '../../common/index.dart';

class ContactsActionAdd extends StatelessWidget {
  const ContactsActionAdd({
    Key key,
    this.contactDetails,
    this.leadDetails,
  }) : super(key: key);

  final ContactDetails contactDetails;
  final LeadDetails leadDetails;

  @override
  Widget build(BuildContext context) {
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    final menuDataList = [
      MenuData(
          Icons.alarm,
          (context, menuData) => navigate(
                      context,
                      ActionTileEvent(
                        title: 'Add Log Response',
                        contactDetails: contactDetails,
                        leadDetails: leadDetails,
                        type: ActionType.logresponse,
                      ),
                      fullScreen: true)
                  .then((pass) {
                if (pass != null) {
                  if (pass) {
                    scaffold.dispatch(ShowSnakBar('Response Added', context));
                  } else {
                    scaffold.dispatch(
                        ShowSnakBar('Error Adding Response', context));
                  }
                }
              }),
          labelText: 'Add Response'),
      MenuData(
          Icons.note_add,
          (context, menuData) => navigate(
                      context,
                      ActionTileEvent(
                        title: 'New Note',
                        contactDetails: contactDetails,
                        leadDetails: leadDetails,
                        type: ActionType.note,
                      ),
                      fullScreen: true)
                  .then((pass) {
                if (pass != null) {
                  if (pass) {
                    scaffold.dispatch(ShowSnakBar('Note Added', context));
                  } else {
                    scaffold
                        .dispatch(ShowSnakBar('Error Adding Note', context));
                  }
                }
              }),
          labelText: 'Add Note'),
      MenuData(
        FontAwesomeIcons.calendarCheck,
        (context, menuData) => navigate(
                context,
                ActionTileEvent(
                  title: 'New Follow Up',
                  contactDetails: contactDetails,
                  leadDetails: leadDetails,
                  type: ActionType.followup,
                ),
                fullScreen: true)
            .then((pass) {
          if (pass != null) {
            if (pass) {
              scaffold.dispatch(ShowSnakBar('Follow Up Added', context));
            } else {
              scaffold.dispatch(ShowSnakBar('Error Adding Follow Up', context));
            }
          }
        }),
        labelText: 'Add Follow Up',
      )
    ];
    return FabMenu(
      menus: menuDataList,
      mainButtonBackgroundColor: Theme.of(context).accentColor,
      mainIcon: Icons.add,
    );
  }
}
