import 'package:flutter/material.dart';
import 'package:flutter_google_places/flutter_google_places.dart';
import 'package:google_maps_webservice/places.dart';

import '../../../data/classes/address.dart';
import '../../../data/utils/openMaps.dart';


class AddressTile extends StatelessWidget {
  AddressTile({
    @required this.address,
    this.label,
    this.icon,
    this.addressPressed,
  });

  final String label;
  final IconData icon;
  final Address address;
  final VoidCallback addressPressed;


  @override
  Widget build(BuildContext context) {
    if (address == null || address.raw().isEmpty) {
      return ListTile(
          leading: Icon(icon ?? Icons.map),
          title: Text(
            label ?? 'Address',
            textScaleFactor: 1,
          ),
          subtitle: Text(
            "No Address Found",
            textScaleFactor: 1,
          ));
    }
    return ListTile(
      leading: Icon(icon ?? Icons.map),
      title: Text(
        label ?? 'Address',
        textScaleFactor: 1,
      ),
      subtitle: Text(
        address.toString(),
        textScaleFactor: 1,
        maxLines: 2,
      ),
      onTap: () {
        if (addressPressed != null) addressPressed();
        return openMaps(context, address.raw(), googleMaps: true);
      },
    );
  }
}

final String kGoogleApiKey = 'AIzaSyCtcngbIjoAaR28UZ__Jy7XYm0pJ7KmhsE';

GoogleMapsPlaces _places = GoogleMapsPlaces(apiKey: kGoogleApiKey);

class AddressInputTile extends StatefulWidget {
  final String label;
  final Address address;
  final ValueChanged<Address> addressChanged;

  AddressInputTile({
    this.addressChanged,
    @required this.address,
    this.label,
  });

  @override
  _AddressInputTileState createState() =>
      _AddressInputTileState(address: address);
}

class _AddressInputTileState extends State<AddressInputTile> {
  Mode _mode = Mode.fullscreen;
  _AddressInputTileState({this.address});
  Address address;

  TextEditingController _street, _apartment, _city, _state, _zip;
  final _formKey = GlobalKey<FormState>();
  bool _isEditing = false;
  @override
  void initState() {
    _street = TextEditingController(text: address?.street ?? "");
    _apartment = TextEditingController(text: address?.apartment ?? "");
    _city = TextEditingController(text: address?.city ?? "");
    _state = TextEditingController(text: address?.state ?? "");
    _zip = TextEditingController(text: address?.zip ?? "");
    super.initState();
  }

  Address getAddress() {
    var _address = Address(
      street: _street?.text ?? "",
      apartment: _apartment?.text ?? "",
      city: _city?.text ?? "",
      state: _state?.text ?? "",
      zip: _zip?.text ?? "",
    );
    // if (_address.raw().isEmpty) return null;
    return _address;
  }

  void onError(PlacesAutocompleteResponse response) {
    print(response);
  }

  Future<void> _handlePressButton(BuildContext context) async {
    Prediction p = await PlacesAutocomplete.show(
      context: context,
      apiKey: kGoogleApiKey,
      onError: onError,
      mode: _mode,
      language: "en",
    );

    if (p != null) {
      print(p.description);

      PlacesDetailsResponse detail =
          await _places.getDetailsByPlaceId(p.placeId);

      for (var _item in detail?.result?.addressComponents) {
        if (_item?.types?.contains("street_number") ?? false) {
          _street.text = _item.longName;
        }
        if (_item?.types?.contains("route") ?? false) {
          _street.text += " " + _item.longName;
        }
        if (_item?.types?.contains("locality") ?? false) {
          _city.text = _item.longName;
        }
        if (_item?.types?.contains("administrative_area_level_1") ?? false) {
          _state.text = _item.shortName;
        }
        if (_item?.types?.contains("postal_code") ?? false) {
          _zip.text = _item.shortName;
        }
      }

      setState(() {
        address = getAddress();
      });

      widget.addressChanged(address);
    }
  }

  @override
  Widget build(BuildContext context) {
    if (!_isEditing) {
      if (address == null || address.raw().isEmpty) {
        return ListTile(
          leading: IconButton(
              icon: Icon(Icons.search),
              onPressed: () => _handlePressButton(context)),
          title: Text(
            widget?.label ?? "Address",
            style: Theme.of(context).textTheme?.body1,
          ),
          subtitle: Text("No Address Added"),
          trailing: Icon(Icons.add),
          onTap: () {
            setState(() {
              _isEditing = !_isEditing;
            });
          },
        );
      }
      return ListTile(
        leading: IconButton(
            icon: Icon(Icons.search),
            onPressed: () => _handlePressButton(context)),
        title: Text(
          widget?.label ?? "Address",
          style: Theme.of(context).textTheme?.body1,
        ),
        subtitle: Text(address.toString()),
        trailing: Icon(Icons.edit),
        onTap: () {
          setState(() {
            _isEditing = !_isEditing;
          });
        },
      );
    }

    return ListTile(
      title: Form(
        autovalidate: true,
        key: _formKey,
        child: Column(
          children: <Widget>[
            TextFormField(
              decoration: InputDecoration(labelText: "Street"),
              controller: _street,
            ),
            TextFormField(
              decoration: InputDecoration(labelText: "Apartment"),
              controller: _apartment,
            ),
            TextFormField(
              decoration: InputDecoration(labelText: "City"),
              controller: _city,
            ),
            TextFormField(
              decoration: InputDecoration(labelText: "State"),
              controller: _state,
            ),
            TextFormField(
              decoration: InputDecoration(labelText: "Zip"),
              controller: _zip,
            ),
          ],
        ),
        onChanged: () {
          setState(() {
            address = getAddress();
          });
          widget.addressChanged(address);
        },
      ),
      trailing: Icon(Icons.close),
      onTap: () {
        setState(() {
          _isEditing = !_isEditing;
        });
      },
    );
  }
}
