import 'dart:async';
import 'dart:convert';

import 'package:after_layout/after_layout.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:get_it/get_it.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/unify.dart' as unify;

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/navigate.dart';
import '../../common/action_list_tile.dart';
import '../../common/email_tile.dart';
import '../../common/index.dart';
import '../../common/manage_groups.dart';
import '../../common/phone_tile.dart';
import '../../common/tracker_list.dart';
import '../../dashboard/cards/index.dart';
import '../../marketing/library/start_campaign.dart';
import '../../video/common/quick_send.dart';
import 'add_contact.dart';
import 'index.dart';
import 'list_loans.dart';
import 'notes_list.dart';
import 'timeline_list.dart';

class ContactDetailsScreen extends StatefulWidget {
  ContactDetailsScreen({Key key, @required this.contact}) : super(key: key);

  final Contact contact;

  @override
  _ContactDetailsScreenState createState() => _ContactDetailsScreenState();
}

class _ContactDetailsScreenState extends State<ContactDetailsScreen>
    with AfterLayoutMixin<ContactDetailsScreen> {
  Timer _timer;

  unify.ContactDetails contactDetails;

  final GlobalKey _keyCard = GlobalKey();

  final DateFormat formatter = new DateFormat('MM-dd-yyyy');

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final userInfo = AuthBloc.currentInfo(context);
    final contacts = BlocProvider.of<ContactsBloc>(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);

    contacts.dispatch(UpdateContactDetails(
        contactId: widget.contact.contactId,
        user: userInfo,
        userId: user.userID));

    return Scaffold(
      appBar: AppBar(
        title: Text('Contact Details'),
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.refresh),
            onPressed: () {},
          )
        ],
      ),
      floatingActionButton: ContactsActionAdd(
        contactDetails: contactDetails,
        // leadDetails: leadDetails,
      ),
      body: SingleChildScrollView(
        child: Padding(
          padding: EdgeInsets.all(12),
          child: StreamBuilder<List<Contact>>(
              stream: GetIt.instance<DatabaseService>()
                  .contactsDao
                  .watchContactsById(user.userID, widget.contact.contactId),
              builder: (context, updateContact) {
                if (updateContact.hasData &&
                    updateContact.data[0].contactDetails != null) {
                  final contactInfo = updateContact.data[0];
                  contactDetails = unify.ContactDetails(
                      userInfo.client,
                      json.decode(contactInfo.contactDetails),
                      widget.contact.contactId);
                  return Column(
                    children: <Widget>[
                      _buildContactInfo(userInfo, context),
                      _buildActions(context, scaffold),
                      _buildExtraCard(),
                      //_buildListActions(context)
                    ],
                  );
                } else {
                  return Center(child: CircularProgressIndicator());
                }
              }),
        ),
      ),
    );
  }

  Container _buildListActions(BuildContext context) {
    return Container(
        padding: EdgeInsets.only(bottom: 10),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Padding(
              padding: EdgeInsets.only(bottom: 10),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Container(
                    width: MediaQuery.of(context).size.width / 2 - 18,
                    child: ActionButtonCard(
                      preview: true,
                      title: 'Notes',
                      subtitle: contactDetails.notes.length.toString(),
                      iconData: Icons.note,
                      color: Colors.amber,
                      onTap: () {
                        showModalBottomSheet(
                          context: context,
                          builder: (context) {
                            return Container(
                              padding: EdgeInsets.all(24),
                              child: SafeArea(
                                child: SingleChildScrollView(
                                  child: FutureBuilder<List<unify.ContactNote>>(
                                      future: contactDetails.listNotes(),
                                      builder: (context, snapshot) {
                                        if (!snapshot.hasData ||
                                            snapshot.data == null) {
                                          return Center(
                                              child:
                                                  CircularProgressIndicator());
                                        }
                                        return Column(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.start,
                                          mainAxisSize: MainAxisSize.min,
                                          children: <Widget>[
                                            Text('Notes',
                                                style: Theme.of(context)
                                                    .textTheme
                                                    .title
                                                    .copyWith(
                                                        fontWeight:
                                                            FontWeight.w700)),
                                            if (snapshot.data.isEmpty) ...[
                                              Center(
                                                  child: Text('No Notes Found'))
                                            ] else ...[
                                              for (var note
                                                  in snapshot.data) ...[
                                                ListTile(
                                                  leading: Icon(Icons.note_add,
                                                      color: Colors.orange),
                                                  title: Text(note.note),
                                                  subtitle:
                                                      Text(note.dateCreated),
                                                )
                                              ]
                                            ]
                                          ],
                                        );
                                      }),
                                ),
                              ),
                            );
                          },
                        );
                      },
                    ),
                  ),
                  Container(
                    width: MediaQuery.of(context).size.width / 2 - 18,
                    child: ActionButtonCard(
                      preview: true,
                      title: 'Timeline',
                      subtitle: contactDetails.tracker.length.toString(),
                      iconData: Icons.flag,
                      color: Colors.green,
                      onTap: () {
                        showModalBottomSheet(
                          context: context,
                          builder: (context) {
                            return Container(
                              padding: EdgeInsets.all(24),
                              child: SafeArea(
                                child: SingleChildScrollView(
                                  child: FutureBuilder<List<unify.Tracker>>(
                                      future: contactDetails.listTracker(),
                                      builder: (context, snapshot) {
                                        if (!snapshot.hasData ||
                                            snapshot.data == null) {
                                          return Center(
                                              child:
                                                  CircularProgressIndicator());
                                        }
                                        return Column(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.start,
                                          mainAxisSize: MainAxisSize.min,
                                          children: <Widget>[
                                            Text('Timeline',
                                                style: Theme.of(context)
                                                    .textTheme
                                                    .title
                                                    .copyWith(
                                                        fontWeight:
                                                            FontWeight.w700)),
                                            if (snapshot.data.isEmpty) ...[
                                              Center(
                                                  child:
                                                      Text('No Events Found')),
                                            ] else ...[
                                              for (var event in snapshot
                                                  .data) ...[TrackerTile(event)]
                                            ]
                                          ],
                                        );
                                      }),
                                ),
                              ),
                            );
                          },
                        );
                      },
                    ),
                  ),
                ],
              ),
            ),
            Container(
              width: MediaQuery.of(context).size.width / 2 - 18,
              child: ActionButtonCard(
                preview: true,
                title: 'Loan History',
                subtitle: contactDetails.loans.length.toString(),
                iconData: Icons.folder,
                color: Colors.grey,
                onTap: () {
                  showModalBottomSheet(
                    context: context,
                    builder: (context) {
                      return Container(
                        padding: EdgeInsets.all(24),
                        child: SafeArea(
                          child: SingleChildScrollView(
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              mainAxisSize: MainAxisSize.min,
                              children: <Widget>[
                                Text('Loans',
                                    style: Theme.of(context)
                                        .textTheme
                                        .title
                                        .copyWith(fontWeight: FontWeight.w700)),
                                if (contactDetails.loans.isEmpty) ...[
                                  Center(child: Text('No Loans Found')),
                                ] else ...[
                                  for (var loan in contactDetails.loans) ...[
                                    ListTile(
                                      leading:
                                          Icon(Icons.info, color: Colors.grey),
                                      title: Text(loan.loanNumber),
                                      subtitle: Text(loan.programTitle),
                                    )
                                  ]
                                ]
                              ],
                            ),
                          ),
                        ),
                      );
                    },
                  );
                },
              ),
            ),
          ],
        ));
  }

  Container _buildExtraCard() {
    return Container(
      padding: EdgeInsets.only(bottom: 10),
      child: RaisedChild(Container(
        padding: EdgeInsets.all(24),
        child: Column(
          children: <Widget>[
            ListTile(
              leading: Icon(Icons.star),
              title:
                  Text(contactDetails.companyCategory.name ?? 'No Name Found'),
            ),
            ListTile(
              leading: Icon(Icons.event),
              title: Text('Date Created'),
              subtitle: Text(formatter.format(widget.contact.dateCreated)),
            ),
            ListTile(
              leading: Icon(Icons.event_available),
              title: Text('Date Modified'),
              subtitle: Text(formatter.format(widget.contact.lastUpdated)),
            )
          ],
        ),
      )),
    );
  }

  Container _buildActions(BuildContext context, ScaffoldBloc scaffold) {
    final userInfo = AuthBloc.currentInfo(context);
    return Container(
      padding: EdgeInsets.only(bottom: 10),
      child: RaisedChild(
        Column(
          children: <Widget>[
            ActionListTile(
              title: 'Timeline',
              iconData: Icons.alarm,
              iconColor: Colors.blueGrey,
              onTap: () => navigate(
                  context, TimelineList(contactDetails: contactDetails),
                  fullScreen: true),
              isList: true,
              count: contactDetails.tracker.length,
            ),
            Divider(),
            ActionListTile(
              title: 'Notes',
              iconData: Icons.note_add,
              iconColor: Colors.amber[700],
              onTap: () => navigate(
                  context, NoteList(contactDetails: contactDetails),
                  fullScreen: true),
              isList: true,
              count: contactDetails.notes.length,
            ),
            if (contactDetails.loans.isNotEmpty) ...[
              Divider(),
              ActionListTile(
                title: 'Loan History',
                iconColor: Colors.grey,
                iconData: Icons.folder,
                isList: true,
                count: contactDetails.loans.length,
                onTap: () => navigate(
                    context, ListLoans(loanList: contactDetails.loans)),
              ),
            ],
            Divider(),
            ActionListTile(
              title: 'Contact Groups',
              iconData: Icons.people,
              iconColor: Colors.green,
              onTap: () => navigate(
                  context,
                  ManageGroups(
                      contactGroups: contactDetails.groups,
                      type: 'Contact',
                      userInfo: userInfo,
                      contact: widget.contact)),
            ),
            Divider(),
            ActionListTile(
              title: 'Send Video Email',
              iconData: Icons.send,
              iconColor: Colors.blue,
              onTap: (){
                final contactList = [widget.contact];
                navigate(context, QuickSendVideo(contacts: contactList));
              },
            ),
            Divider(),
            Padding(
              padding: const EdgeInsets.only(bottom: 8.0),
              child: ActionListTile(
                title: 'Start Marketing',
                iconColor: Colors.red,
                iconData: FontAwesomeIcons.bullseye,
                onTap: () {
                  final contactList = [widget.contact];
                  navigate(context, StartCampaignScreen(contacts: contactList));
                },
              ),
            ),
          ],
        ),
      ),
    );
  }

  Padding _buildContactInfo(unify.AccountInfo userInfo, BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 10),
      child: RaisedChild(
        Container(
          padding: EdgeInsets.all(24),
          child: Column(
            children: <Widget>[
              Row(
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  Text('${widget.contact.firstName} ${widget.contact.lastName}',
                      style:
                          TextStyle(fontWeight: FontWeight.w700, fontSize: 18)),
                  GestureDetector(
                    child: Padding(
                      padding: const EdgeInsets.only(left: 8.0),
                      child: Icon(Icons.edit),
                    ),
                    onTap: () => navigate(
                        context, AddEditContact(contact: contactDetails)),
                  )
                ],
              ),
              PhoneTile(
                label: 'Cell Phone',
                number: contactDetails.cellPhone,
                smsPressed: () {
                  contactDetails.addNote(
                      'From My Unify CRM Mobile App: A SMS event has been triggered from cell phone ${userInfo.primaryPhone}, please refer to your text messages for further details on the content of message.');
                },
                callPressed: () {
                  contactDetails.addNote(
                      'From My Unify CRM Mobile App: A phone call event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this call, please create a separate note via the Unify mobile app or website.');
                },
                icon: Icons.phone,
              ),
              PhoneTile(
                label: 'Home Phone',
                number: contactDetails.homePhone,
                smsPressed: () {
                  contactDetails.addNote(
                      'From My Unify CRM Mobile App: A SMS event has been triggered from cell phone ${userInfo.primaryPhone}, please refer to your text messages for further details on the content of message.');
                },
                callPressed: () {
                  contactDetails.addNote(
                      'From My Unify CRM Mobile App: A phone call event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this call, please create a separate note via the Unify mobile app or website.');
                },
                icon: Icons.home,
              ),
              PhoneTile(
                label: 'Office Phone',
                number: contactDetails.officePhone,
                smsPressed: () {
                  contactDetails.addNote(
                      'From My Unify CRM Mobile App: A SMS event has been triggered from cell phone ${userInfo.primaryPhone}, please refer to your text messages for further details on the content of message.');
                },
                callPressed: () {
                  contactDetails.addNote(
                      'From My Unify CRM Mobile App: A phone call event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this call, please create a separate note via the Unify mobile app or website.');
                },
                icon: Icons.work,
              ),
              EmailTile(
                  email: contactDetails.email,
                  label: 'Email Address',
                  icon: Icons.email,
                  emailPressed: () {
                    contactDetails.addNote(
                        'From My Unify CRM Mobile App: An e-mail event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this e-mail, please create a separate note via the Unify mobile app or website.');
                  }),
              ListTile(
                leading: Icon(Icons.map),
                title: Text('Current Address'),
                subtitle: Text(formatAddress(contact: contactDetails)),
              ),
            ],
          ),
        ),
      ),
    );
  }

  String formatAddress({bool current = false, unify.ContactDetails contact}) {
    final address = contact.currentAddress;

    if (address == null || address.raw().isEmpty) {
      return 'No Address Found';
    }
    return '${address.street} ${address.city}\n${address.state} ${address.zip}';
  }

  @override
  void afterFirstLayout(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final userInfo = AuthBloc.currentInfo(context);
    BlocProvider.of<ContactsBloc>(context).dispatch(UpdateContactDetails(
        contactId: widget.contact.contactId,
        user: userInfo,
        userId: user.userID));
  }
}
