import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:unify_api/unify.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/utils/navigate.dart';
import '../../common/index.dart';

class NoteList extends StatelessWidget {
  NoteList({Key key, this.contactDetails, this.leadDetails}) : super(key: key);

  final ContactDetails contactDetails;
  final LeadDetails leadDetails;

  @override
  Widget build(BuildContext context) {
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);

    return Scaffold(
      appBar: AppBar(
        title: Text('Notes'),
      ),
      floatingActionButton: FloatingActionButton(
        child: Icon(Icons.add),
        onPressed: () {
          navigate(
                  context,
                  ActionTileEvent(
                    type: ActionType.note,
                    title: 'New Note',
                    contactDetails: contactDetails,
                    leadDetails: leadDetails,
                  ),
                  fullScreen: true)
              .then(
            (pass) {
              if (pass != null) {
                if (pass) {
                  scaffold.dispatch(ShowSnakBar('Note Added', context));
                } else {
                  scaffold.dispatch(ShowSnakBar('Error Adding Note', context));
                }
              }
            },
          );
        },
      ),
      body: Container(
        child: SafeArea(
          child: SingleChildScrollView(
            child: FutureBuilder<List<dynamic>>(
              future: _getNotes(),
              builder: (context, snapshot) {
                if (!snapshot.hasData || snapshot.data == null) {
                  return Center(child: CircularProgressIndicator());
                } else {
                  return Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      if (snapshot.data.isEmpty) ...[
                        Center(child: Text('No Notes Found'))
                      ] else ...[
                        for (var note in snapshot.data) ...[
                          ListTile(
                            leading: Icon(Icons.note_add, color: Colors.orange),
                            title: Text(note.note),
                            subtitle: Text(note.dateCreated),
                          )
                        ]
                      ]
                    ],
                  );
                }
              },
            ),
          ),
        ),
      ),
    );
  }

  Future<List> _getNotes() {
    if (contactDetails != null) {
      return contactDetails.listNotes();
    }
    if (leadDetails != null) {
      return leadDetails.listNotes();
    }
    return null;
  }
}
