import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/blocs/search/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import 'detailed_screen.dart';
import 'detailed_tile.dart';
import 'more_button.dart';

class ContactRowTile extends StatefulWidget {
  const ContactRowTile({
    Key key,
    @required Contact contact,
    @required ViewType type,
    this.border,
    this.iconColor,
    this.onTap,
    this.onLongPress,
    this.favorite = false,
    this.favId,
    this.inGroup = false,
    this.groupId,
    this.itemId,
  })  : _contact = contact,
        _type = type,
        super(key: key);

  final Contact _contact;
  final ViewType _type;
  final BoxDecoration border;
  final Color iconColor;
  final VoidCallback onTap, onLongPress;
  final bool favorite, inGroup;
  final int favId;
  final String groupId, itemId;

  @override
  _ContactRowTileState createState() => _ContactRowTileState();
}

class _ContactRowTileState extends State<ContactRowTile> {
  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final info = AuthBloc.currentInfo(context);
    final _client = info.client;
    final searchBloc = BlocProvider.of<SearchBloc>(context);
    switch (widget._type) {
      case ViewType.detailed:
        return GestureDetector(
          onTap: () {
            Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) =>
                        ContactDetailsScreen(contact: widget._contact)));
          },
          child: Slidable(
            actionPane: SlidableDrawerActionPane(),
            actionExtentRatio: 0.25,
            child: DetailedTile(
              contact: widget._contact,
              name: '${widget._contact.firstName} ${widget._contact.lastName}',
              subtitle: widget._contact.lastActivity,
            ),
            actions: <Widget>[
              if(widget.inGroup)
                IconSlideAction(
                caption: 'Remove',
                color: Colors.red,
                icon: Icons.remove_circle_outline,
                onTap: () async {
                  final contactGroups = await _client
                      .getContactDetails(widget._contact.contactId)
                      .then((details) {
                    final grps = details.groups;
                    for (var group in grps) {
                    if (group.id == widget.groupId) {
                      grps.remove(group);
                    }
                    return grps;
                  }
                  });
                  print('TEST');
                  await GetIt.instance<DatabaseService>()
                      .groupItemDao
                      .removeItem(widget._contact.contactId, user.userID, widget.groupId);
                  await _client.editContact(widget._contact.contactId,
                      groups: contactGroups,
                      firstName: widget._contact.firstName,
                      lastName: widget._contact.lastName);
                  
                },
              ),
              if(!widget.inGroup)
                IconSlideAction(
                  caption: 'Delete',
                  color: Colors.red,
                  icon: Icons.delete,
                  onTap: () {
                    GetIt.instance<DatabaseService>()
                        .contactsDao
                        .deleteContact(user.userID, widget._contact.contactId);
                    searchBloc.dispatch(TextChanged(text: '', user: info));
                    _client.deleteContact(widget._contact.contactId);
                  },
                )
            ],
            secondaryActions: <Widget>[
            if (!widget.favorite)
              IconSlideAction(
                caption: 'Favorite',
                color: Colors.green,
                icon: Icons.star,
                onTap: () {
                  GetIt.instance<DatabaseService>().favoritesDao.addItem(
                      'contact', widget._contact.contactId, user.userID);
                },
              ),
            if (widget.favorite)
              IconSlideAction(
                caption: 'Unfavorite',
                color: Colors.green,
                icon: Icons.star_border,
                onTap: () {
                  GetIt.instance<DatabaseService>().favoritesDao.removeItem(widget.favId, user.userID);
                },
              ),
            
          ],
          ),
        );
        break;
      case ViewType.dense:
        return Slidable(
          actionPane: SlidableDrawerActionPane(),
          actionExtentRatio: 0.25,
          child: Container(
            decoration: widget.border,
            child: ListTile(
              onLongPress: () {},
              onTap: () {
                Navigator.push(
                    context,
                    MaterialPageRoute(
                        builder: (context) =>
                            ContactDetailsScreen(contact: widget._contact)));
              },
              title: Text(
                  '${widget._contact.firstName}, ${widget._contact.lastName}'),
              subtitle: Text(widget._contact.lastActivity),
              leading: Icon(
                Icons.account_circle,
                size: 34,
                color: widget.iconColor,
              ),
              trailing: MoreButton(
                email: widget._contact.email,
                cell: widget._contact.cellPhone,
                office: widget._contact.officePhone,
                home: widget._contact.homePhone,
              ),
            ),
          ),
          actions: <Widget>[
            // if (widget.favorite)
            //   IconSlideAction(
            //     caption: 'Remove',
            //     color: Colors.red,
            //     icon: Icons.delete,
            //     onTap: () {
            //       GetIt.instance<DatabaseService>()
            //           .favoritesDao
            //           .removeItem(widget.favId, user.userID);
            //     },
            //   )
          ],
          secondaryActions: <Widget>[
            if (!widget.favorite)
              IconSlideAction(
                caption: 'Favorite',
                color: Colors.green,
                icon: Icons.star,
                onTap: () {
                  GetIt.instance<DatabaseService>().favoritesDao.addItem(
                      'contact', widget._contact.contactId, user.userID);
                },
              ),
            if (widget.favorite)
              IconSlideAction(
                caption: 'Unfavorite',
                color: Colors.green,
                icon: Icons.star_border,
                onTap: () {
                  GetIt.instance<DatabaseService>().favoritesDao.removeItem(widget.favId, user.userID);
                },
              ),
            
          ],
        );
        break;
    }
    return Container();
  }
}
