import 'package:auto_size_text/auto_size_text.dart';
import 'package:flip_card/flip_card.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:get_it/get_it.dart';
import 'package:hive/hive.dart';
import 'package:hive_flutter/hive_flutter.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../common/raised_card.dart';
import '../common/action_button_builder.dart';

class ActionButtonCard extends StatefulWidget {
  static const String cardKey = 'action_storage';

  static const BoxConstraints constraints = BoxConstraints(
    minHeight: 175.0,
    minWidth: 175.0,
    maxHeight: 175,
    maxWidth: 175,
  );

  ActionButtonCard(
      {this.title = 'Add',
      this.subtitle = 'Create New Action',
      this.color = Colors.teal,
      this.iconData = Icons.add,
      this.onTap,
      this.id,
      this.fontawesome = false,
      this.preview = false});

  final String title, subtitle;
  final IconData iconData;
  final Color color;
  final VoidCallback onTap;
  final int id;
  final bool preview;
  final bool fontawesome;

  @override
  _ActionButtonCardState createState() => _ActionButtonCardState();
}

class _ActionButtonCardState extends State<ActionButtonCard>
    with AutomaticKeepAliveClientMixin<ActionButtonCard> {
  @override
  bool get wantKeepAlive => false;
  //LocalStorage storage;
  String _title, _subtitle, fontFam, fontPack;
  IconData _iconData;
  int _color;
  Box box;
  int icon;
  GlobalKey<FlipCardState> cardKey = GlobalKey<FlipCardState>();

  @override
  void initState() {
    init();
    super.initState();
  }

  Future init() async {
    return box = await Hive.openBox('action_storage_${widget?.id ?? 0}');
  }

  @override
  Widget build(BuildContext context) {
    print(FontAwesomeIcons.calculator.fontFamily);
    final _user = AuthBloc.currentUser(context);
    final _info = AuthBloc.currentInfo(context);
    final _isVideo = widget.title == 'Record' || widget.title == 'Quick Send';
    if(widget.fontawesome){
      fontFam = widget.iconData.fontFamily;
      fontPack = widget.iconData.fontPackage;
    }else{
      fontFam = 'MaterialIcons';
      fontPack = widget.iconData.fontPackage;
    }
    return FutureBuilder(
        future: init(),
        builder: (context, snapshot) {
          if (box == null || !snapshot.hasData) {
            return RaisedChild(Container(
                padding: EdgeInsets.all(34),
                child: CircularProgressIndicator()));
          }
          if (box.isOpen) {
            return GestureDetector(
              onTap: widget.onTap,
              onDoubleTap: () => {
                if(!widget.preview)
                  cardKey.currentState.toggleCard()
              },
              child: FlipCard(
                flipOnTouch: false,
                speed: 450,
                key: cardKey,
                front: RaisedChild(
                  Stack(
                    children: <Widget>[
                      Opacity(
                        opacity: !_isVideo || _isVideo && _info.videoEnabled
                            ? 1
                            : 0.5,
                        child: Padding(
                          padding: EdgeInsets.all(20),
                          child: WatchBoxBuilder(
                              box: box,
                              builder: (context, box) {
                                return ActionButtonUI(
                                    box: box,
                                    color: widget.color.value,
                                    title: widget.title,
                                    subtitle: widget.subtitle,
                                    fontFam: fontFam,
                                    fontPack: fontPack,
                                    icon: widget.iconData.codePoint);
                              }),
                        ),
                      ),
                      if (_isVideo && !_info.videoEnabled) ...[
                        Center(
                          child: Text(
                            'Disabled',
                            style: TextStyle(
                                fontWeight: FontWeight.w600, fontSize: 25),
                          ),
                        )
                      ]
                    ],
                  ),
                ),
                back: RaisedChild(Column(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: <Widget>[
                    FlatButton.icon(
                      label: Text('Delete Widget'),
                      icon: Icon(Icons.delete),
                      onPressed: () {
                        deleteWidget(_user, widget.id);
                      },
                    ),
                    FlatButton.icon(
                      icon: Icon(Icons.settings),
                      label: Text('Settings'),
                      onPressed: () {
                        cardKey.currentState.toggleCard();
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (context) => BuildActionScreen(
                                      id: widget.id ?? 0,
                                    )));
                      },
                    ),
                  ],
                )),
              ),
            );
          }
        });
  }
}

void deleteWidget(User user, int id) async {
  await GetIt.instance<DatabaseService>().homeDao.removeItem(id, user.userID);
}

class ActionButtonUI extends StatelessWidget {
  ActionButtonUI({
    Key key,
    @required this.box,
    this.title,
    this.subtitle,
    this.icon,
    this.color,
    this.fontFam,
    this.fontPack,
    this.load = true,
    
  }) : super(key: key);

  final Box box;
  String title, subtitle;
  int icon;
  int color;
  String fontFam, fontPack;
  final bool load;


  void loadValues() {
    title = box.get('action_title') ?? 'Add';
    subtitle = box.get('action_subtitle') ?? 'Create New Action';
    icon = box.get('action_icon') ?? Icons.add.codePoint;
    color = box.get('action_color') ?? Colors.teal.value;
  }

  @override
  Widget build(BuildContext context) {
    if (box.isNotEmpty && load == true) {
      loadValues();
    }
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Material(
          color: Color(color ?? Colors.teal.value),
          shape: CircleBorder(),
          child: Padding(
            padding: EdgeInsets.all(16),
            child: Icon(
                IconData(icon ?? Icons.add.codePoint,
                    fontFamily: fontFam, fontPackage: fontPack),
                color: Colors.white,
                size: MediaQuery.of(context).size.width < 350 ? 20 : 30),
          ),
        ),
        Container(
          padding: EdgeInsets.only(top: 16),
          child: AutoSizeText(
            title ?? 'Add',
            maxLines: 1,
            textScaleFactor: 1,
            style: TextStyle(fontWeight: FontWeight.w700, fontSize: 24),
          ),
        ),
        Container(
          child: subtitle == null
              ? null
              : AutoSizeText(
                  subtitle ?? 'Create new action',
                  textScaleFactor: 1,
                  style: Theme.of(context).textTheme.subhead,
                  maxLines: 1,
                ),
        ),
      ],
    );
  }
}
