import 'package:flutter/material.dart';

import '../../../data/services/database/database.dart';
import '../../../data/utils/date_formatter.dart';
import '../../contacts/common/detailed_tile.dart';
import '../../contacts/common/more_button.dart';

class DetailedLeadTile extends StatelessWidget {
  DetailedLeadTile({
    Key key,
    @required Lead lead,
    this.name,
    this.subtitle,
  })  : _lead = lead,
        super(key: key);

  final Lead _lead;
  final String name;
  final String subtitle;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.all(8),
      child: LayoutBuilder(
        builder: (context, dimens) => SizedBox(
          height: 100,
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              Icon(Icons.people, size: 45),
              Expanded(
                  child: Padding(
                      padding: EdgeInsets.only(left: 10, right: 10),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          Text(
                            name,
                            style: Theme.of(context).textTheme.title,
                            maxLines: 1,
                          ),
                          Padding(padding: EdgeInsets.only(bottom: 6)),
                          Text(
                            subtitle,
                            style: Theme.of(context).textTheme.subtitle,
                            maxLines: 1,
                          ),
                          if (dimens.maxWidth < 450 && dimens.maxWidth > 340)
                            _buildMore(),
                        ],
                      ))),
              if (dimens.maxWidth >= 450) _buildMore(),
              Column(
                crossAxisAlignment: CrossAxisAlignment.end,
                children: <Widget>[
                  MoreButton(
                    cell: _lead.cellPhone,
                    email: _lead.email,
                    home: _lead.homePhone,
                    office: _lead.officePhone,
                  ),
                ],
              )
            ],
          ),
        ),
      ),
    );
  }

  Row _buildMore() {
    return Row(
      children: <Widget>[
        TileAction(
          label: formatDateCustom(_lead.dateCreated, format: "MM-dd-yy"),
          tooltip: "Date Created",
          avatar: Icon(Icons.event),
        ),
        TileAction(
          label: formatDateCustom(_lead.lastUpdated, format: "MM-dd-yy"),
          tooltip: "Date Modified",
          avatar: Icon(Icons.event),
        ),
      ],
    );
  }
}
