import 'dart:convert';

import 'package:after_layout/after_layout.dart';
import 'package:expandable/expandable.dart';
import 'package:flip_card/flip_card.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:get_it/get_it.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/unify.dart' as unify;

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/is_empty.dart';
import '../../../data/utils/navigate.dart';
import '../../common/action_list_tile.dart';
import '../../common/email_tile.dart';
import '../../common/manage_groups.dart';
import '../../common/phone_tile.dart';
import '../../common/raised_card.dart';
import '../../contacts/common/notes_list.dart';
import '../../contacts/common/timeline_list.dart';
import '../../marketing/library/start_campaign.dart';
import '../../video/common/quick_send.dart';
import 'add_lead.dart';

class LeadsDetailScreen extends StatefulWidget {
  LeadsDetailScreen({Key key, @required this.lead}) : super(key: key);

  final Lead lead;

  @override
  _LeadsDetailScreenState createState() => _LeadsDetailScreenState();
}

class _LeadsDetailScreenState extends State<LeadsDetailScreen>
    with AfterLayoutMixin<LeadsDetailScreen> {
  unify.LeadDetails leadDetails;

  final DateFormat formatter = new DateFormat('MM-dd-yyyy');

  final money = NumberFormat.simpleCurrency();
  Lead get lead => widget.lead;
  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final userInfo = AuthBloc.currentInfo(context);

    return Scaffold(
      appBar: AppBar(
        title: Text('Lead Details'),
      ),
      body: SingleChildScrollView(
        child: Padding(
          padding: EdgeInsets.all(12),
          child: StreamBuilder<List<Lead>>(
            stream: GetIt.instance<DatabaseService>()
                .leadsDao
                .watchLeadsById(user.userID, widget.lead.leadId),
            builder: (context, snapshot) {
              if (snapshot.hasData && snapshot.data[0].leadDetails != null) {
                final leadInfo = snapshot.data[0];
                leadDetails = unify.LeadDetails(userInfo.client,
                    json.decode(leadInfo.leadDetails), widget.lead.leadId);
                return Column(
                  children: <Widget>[
                    _buildContactInfo(userInfo, context),
                    _buildActions(context),
                    _buildExtraCard(),
                    //_buildListActions(context),
                  ],
                );
              } else {
                return Center(child: CircularProgressIndicator());
              }
            },
          ),
        ),
      ),
    );
  }

  FlipCard _buildContactInfo(unify.AccountInfo userInfo, BuildContext context) {
    return FlipCard(
      flipOnTouch: false,
      speed: 400,
      front: Padding(
        padding: const EdgeInsets.only(bottom: 10),
        child: RaisedChild(
          Container(
            padding: EdgeInsets.all(24),
            child: Column(
              children: <Widget>[
                Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  children: <Widget>[
                    Text('${widget.lead.firstName} ${widget.lead.lastName}',
                        style: TextStyle(
                            fontWeight: FontWeight.w700, fontSize: 18)),
                    GestureDetector(
                      child: Padding(
                        padding: EdgeInsets.only(left: 8),
                        child: Icon(Icons.edit),
                      ),
                      onTap: () =>
                          navigate(context, AddEditLead(lead: leadDetails)),
                    ),
                  ],
                ),
                PhoneTile(
                  label: 'Cell Phone',
                  number: leadDetails.cellPhone,
                  smsPressed: () {
                    leadDetails.addNote(
                        'From My Unify CRM Mobile App: A SMS event has been triggered from cell phone ${userInfo.primaryPhone}, please refer to your text messages for further details on the content of message.');
                  },
                  callPressed: () {
                    leadDetails.addNote(
                        'From My Unify CRM Mobile App: A phone call event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this call, please create a separate note via the Unify mobile app or website.');
                  },
                  icon: Icons.phone,
                ),
                PhoneTile(
                  label: 'Home Phone',
                  number: leadDetails.homePhone,
                  smsPressed: () {
                    leadDetails.addNote(
                        'From My Unify CRM Mobile App: A SMS event has been triggered from cell phone ${userInfo.primaryPhone}, please refer to your text messages for further details on the content of message.');
                  },
                  callPressed: () {
                    leadDetails.addNote(
                        'From My Unify CRM Mobile App: A phone call event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this call, please create a separate note via the Unify mobile app or website.');
                  },
                  icon: Icons.home,
                ),
                PhoneTile(
                  label: 'Office Phone',
                  number: leadDetails.officePhone,
                  smsPressed: () {
                    leadDetails.addNote(
                        'From My Unify CRM Mobile App: A SMS event has been triggered from cell phone ${userInfo.primaryPhone}, please refer to your text messages for further details on the content of message.');
                  },
                  callPressed: () {
                    leadDetails.addNote(
                        'From My Unify CRM Mobile App: A phone call event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this call, please create a separate note via the Unify mobile app or website.');
                  },
                  icon: Icons.work,
                ),
                EmailTile(
                    email: leadDetails.email,
                    label: 'Email Address',
                    icon: Icons.email,
                    emailPressed: () {
                      leadDetails.addNote(
                          'From My Unify CRM Mobile App: An e-mail event has been triggered from cell phone ${userInfo.primaryPhone}, if you would like to add additional details about this e-mail, please create a separate note via the Unify mobile app or website.');
                    }),
                ListTile(
                  leading: Icon(Icons.map),
                  title: Text('Current Address'),
                  subtitle:
                      Text(formatAddress(current: true, lead: leadDetails)),
                ),
                ListTile(
                  leading: Icon(Icons.map),
                  title: Text('Property Address'),
                  subtitle: Text(formatAddress(lead: leadDetails)),
                ),
              ],
            ),
          ),
        ),
      ),
      back: RaisedChild(Container(
        //height: _keyCard.currentContext.size.height,
        child: Padding(
          padding: EdgeInsets.all(24),
          child: Column(
            children: <Widget>[
              Row(
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  Text('Edit Contact',
                      style:
                          TextStyle(fontWeight: FontWeight.w700, fontSize: 24)),
                ],
              ),
              TextFormField(
                decoration: InputDecoration(labelText: 'First Name'),
                initialValue: widget.lead.firstName,
              ),
              TextFormField(
                decoration: InputDecoration(labelText: 'Last Name'),
                initialValue: widget.lead.lastName,
              ),
              TextFormField(
                decoration: InputDecoration(labelText: 'Email'),
                initialValue: widget.lead.email,
              ),
              ExpansionTile(
                title: Text('Phone Numbers'),
                leading: Icon(Icons.phone),
                children: <Widget>[],
              ),
              ExpansionTile(
                title: Text('Address info'),
                leading: Icon(Icons.map),
                children: <Widget>[],
              )
            ],
          ),
        ),
      )),
    );
  }

  Container _buildActions(BuildContext context) {
    final userInfo = AuthBloc.currentInfo(context);
    return Container(
      padding: EdgeInsets.only(bottom: 10),
      child: RaisedChild(Column(
        children: <Widget>[
          ActionListTile(
            title: 'Timeline',
            iconData: Icons.alarm,
            iconColor: Colors.blueGrey,
            count: leadDetails.tracker.length,
            isList: true,
            onTap: () => navigate(
                context, TimelineList(leadDetails: leadDetails),
                fullScreen: true),
          ),
          Divider(),
          ActionListTile(
            title: 'Notes',
            iconData: Icons.note_add,
            iconColor: Colors.amber[700],
            count: leadDetails.notes.length,
            isList: true,
            onTap: () => navigate(context, NoteList(leadDetails: leadDetails),
                fullScreen: true),
          ),
          Divider(),
          Padding(
            padding: const EdgeInsets.only(left: 20.0, right: 25),
            child: ExpandablePanel(
                header: ListTile(
                  leading: Icon(Icons.folder),
                  title: Text('Loans'),
                ),
                expanded: Column(
                  children: <Widget>[
                    if (leadDetails.currentLoan != null)
                      ActionListTile(
                        iconData: Icons.folder,
                        iconColor: Colors.grey,
                        title: 'Current Loan',
                        onTap: () {},
                      ),
                    if (leadDetails.secondLoan != null)
                      ActionListTile(
                        iconData: Icons.folder,
                        iconColor: Colors.grey,
                        title: 'Second Loan',
                        onTap: () {},
                      ),
                    if (leadDetails.requestedLoan != null)
                      ActionListTile(
                        iconData: Icons.folder,
                        iconColor: Colors.grey,
                        title: 'Requested Loan',
                        onTap: () {},
                      )
                  ],
                )),
          ),
          Divider(),
          ActionListTile(
            title: 'Lead Groups',
            iconData: Icons.people,
            iconColor: Colors.green[700],
            onTap: () => navigate(
                context,
                ManageGroups(
                  leadGroups: leadDetails.groups,
                  type: 'Lead',
                  lead: widget.lead,
                  userInfo: userInfo,
                )),
          ),
          Divider(),
          ActionListTile(
            title: 'Send Video Email',
            iconData: Icons.send,
            iconColor: Colors.blue,
            onTap: () {
              final leadList = [lead];
              navigate(context, QuickSendVideo(leads: leadList));
            },
          ),
          Divider(),
          Padding(
            padding: const EdgeInsets.only(bottom: 8.0),
            child: ActionListTile(
                title: 'Start Marketing',
                iconData: FontAwesomeIcons.bullseye,
                iconColor: Colors.red,
                onTap: () {
                  final leadList = [lead];
                  navigate(context, StartCampaignScreen(leads: leadList));
                }),
          ),
        ],
      )),
    );
  }

  Container _buildExtraCard() {
    return Container(
      padding: EdgeInsets.only(bottom: 10),
      child: RaisedChild(Container(
        padding: EdgeInsets.all(24),
        child: Column(
          children: <Widget>[
            ListTile(
              leading: Icon(FontAwesomeIcons.percent),
              title: Text('Credit Range'),
              subtitle: Text(isEmpty(
                  string: leadDetails.creditRange,
                  returnStr: 'No Credit Found')),
            ),
            ListTile(
              leading: Icon(Icons.attach_money),
              title: Text('Annual Gross Income'),
              subtitle: Text(money.format(leadDetails.annualGrossIncome)),
            ),
            ListTile(
              leading: Icon(Icons.phone),
              title: Text('Best Call Time'),
              subtitle: Text(isEmpty(
                  string: leadDetails.bestCallTime,
                  returnStr: 'No Time Specified')),
            ),
            ListTile(
              leading: Icon(Icons.folder),
              title: Text('Recent Bankruptcy'),
              subtitle: Text(leadDetails.recentBankruptcy.toString()),
            ),
            ListTile(
              leading: Icon(Icons.event),
              title: Text('Date Created'),
              subtitle: Text(formatter.format(widget.lead.dateCreated)),
            ),
            ListTile(
              leading: Icon(Icons.event_available),
              title: Text('Date Modified'),
              subtitle: Text(formatter.format(widget.lead.lastUpdated)),
            )
          ],
        ),
      )),
    );
  }

  String formatAddress({bool current = false, unify.LeadDetails lead}) {
    final address = (current) ? lead.currentAddress : lead.propertyAddress;

    if (address == null || address.raw().isEmpty) {
      return 'No Address Found';
    }
    return '${address.street} ${address.city}\n${address.state} ${address.zip}';
  }

  @override
  void afterFirstLayout(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    final userInfo = AuthBloc.currentInfo(context);
    BlocProvider.of<LeadsBloc>(context).dispatch(UpdateLeadDetails(
      leadId: widget.lead.leadId,
      user: userInfo,
      userId: user.userID,
    ));
  }
}
