import 'package:example_db/data/blocs/auth/auth_bloc.dart';
import 'package:example_db/data/classes/enum_list.dart';
import 'package:example_db/data/services/database/database.dart';
import 'package:example_db/ui/leads/common/tile.dart';
import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';

class LeadFavoritesScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    return Scaffold(
      body: SafeArea(
        child: Container(
          child: StreamBuilder<List<Favorite>>(
            stream: GetIt.instance<DatabaseService>()
                .favoritesDao
                .watchFavoriteByType(user.userID, 'lead'),
            builder: (context, list) {
              if (!list.hasData || list.data == null) {
                return Center(child: CircularProgressIndicator());
              } else {
                if (list.data.isEmpty) {
                  return Center(child: Text('No Favorites'));
                }
                return SingleChildScrollView(
                  child: Column(
                    children: <Widget>[
                      for (var fav in list.data) ...[
                        FutureBuilder<Lead>(
                          future: GetIt.instance<DatabaseService>()
                              .leadsDao
                              .getLead(user.userID, fav.typeId),
                          builder: (context, lead){
                            if(lead.data != null && lead.hasData){
                              return Container(
                                child: LeadRowTile(
                                  lead: lead.data,
                                  type: ViewType.dense,
                                  border: _buildBorder(Colors.black),
                                  iconColor: Colors.black,
                                  favorite: true,
                                  favId: fav.favoriteId,
                                ),
                              );
                            }
                            return Container();
                          },
                        )
                      ]
                    ],
                  ),
                );
              }
            },
          ),
        ),
      ),
    );
  }

  BoxDecoration _buildBorder(Color color) {
    return BoxDecoration(
        border: Border(
      left: BorderSide(
        color: color,
        width: 5.0,
      ),
    ));
  }

}
