import 'package:example_db/data/services/database/database.dart' as db;
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:unify_api/unify.dart' as api;

import '../../../data/blocs/bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/utils/navigate.dart';
import '../../common/raised_card.dart';
import 'tile.dart';

class LeadGroupsScreen extends StatelessWidget {
  List<api.LeadGroup> leadGroups;

  static refresh(BuildContext context) {
    final leads = BlocProvider.of<LeadsBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    return leads.dispatch(
        RefreshLeads(user: _user, scaffold: scaffold, context: context));
  }

  @override
  Widget build(BuildContext context) {
    final leads = BlocProvider.of<LeadsBloc>(context);
    final user = AuthBloc.currentInfo(context);
    final client = user.client;

    return Scaffold(
      body: SafeArea(
        child: StreamBuilder<LeadsState>(
          stream: leads.state,
          builder: (context, snapshot) {
            final _state = snapshot.data;
            if (_state is LeadsLoadingState) {
              return Container(
                  child: Center(child: CircularProgressIndicator()));
            }
            return StreamBuilder<List<db.LeadGroup>>(
              stream: GetIt.instance<db.DatabaseService>()
                  .leadGroupDao
                  .watchGroupsByUser(user.id),
              builder: (context, list) {
                if (list.hasData && list.data.isNotEmpty) {
                  final _groups = list.data;

                  return Scrollbar(
                    child: SingleChildScrollView(
                      child: Wrap(
                        children: <Widget>[
                          for (var group in _groups) ...[
                            Padding(
                              padding:
                                  const EdgeInsets.symmetric(horizontal: 4.0),
                              child: ActionChip(
                                avatar: CircleAvatar(
                                  child: Text('${group.count}'),
                                ),
                                label: Text(group.groupName),
                                onPressed: () => navigate(context,
                                    ViewGroupLeads(group: group, user: user)),
                              ),
                            ),
                          ],
                        ],
                      ),
                    ),
                  );
                }
                if (_state is LeadsLoadingState) {
                  return Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: <Widget>[
                      Text('No Groups Found'),
                      SizedBox(height: 20),
                      CircularProgressIndicator()
                    ],
                  );
                }
                return Center(child: Text('No Groups Founds'));
              },
            );
          },
        ),
      ),
    );
  }
}

class ViewGroupLeads extends StatelessWidget {
  const ViewGroupLeads({Key key, this.group, this.user}) : super(key: key);

  final db.LeadGroup group;
  final api.AccountInfo user;

  static refresh(BuildContext context, db.LeadGroup group) {
    final leads = BlocProvider.of<LeadsBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    return leads.dispatch(UpdateGroupItems(group: group, user: _user));
  }

  @override
  Widget build(BuildContext context) {
    final leads = BlocProvider.of<LeadsBloc>(context);
    leads.dispatch(UpdateGroupItems(group: group, user: user));
    return Scaffold(
      appBar: AppBar(
        title: Text('Leads'),
      ),
      body: SafeArea(
        child: Container(
          child: StreamBuilder<List<db.GroupItem>>(
            stream: GetIt.instance<db.DatabaseService>()
                .groupItemDao
                .watchItemsByGroup(group.groupId, user.id),
            builder: (context, list) {
              if (list.hasData && list.data.isNotEmpty) {
                return Scrollbar(
                  child: RefreshIndicator(
                    onRefresh: () async {
                      refresh(context, group);
                      await Future.delayed(Duration(seconds: 3));
                    },
                    child: SingleChildScrollView(
                      child: Column(
                        children: <Widget>[
                          for (var lead in list.data) ...[
                            FutureBuilder<db.Lead>(
                              future: GetIt.instance<db.DatabaseService>()
                                  .leadsDao
                                  .getLead(user.id, lead.itemId),
                              builder: (context, lead) {
                                if (lead.data != null && lead.hasData) {
                                  return Container(
                                    child: LeadRowTile(
                                      inGroup: true,
                                      groupId: group.groupId,
                                      lead: lead.data,
                                      type: ViewType.detailed,
                                    ),
                                  );
                                }
                                return Container();
                              },
                            )
                          ]
                        ],
                      ),
                    ),
                  ),
                );
              }
              return Container();
            },
          ),
        ),
      ),
    );
  }
}
