import 'package:flutter/material.dart';
import 'package:get_it/get_it.dart';

import '../../../data/blocs/bloc.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/debouncer.dart';

class SearchLeads extends StatefulWidget {
  final List<Lead> selected;

  const SearchLeads({
    Key key,
    @required this.selected,
  }) : super(key: key);
  @override
  _SearchLeadsState createState() => _SearchLeadsState();
}

class _SearchLeadsState extends State<SearchLeads> {
  final _controller = TextEditingController();
  String _query = '';
  final _debouncer = Debouncer();

  @override
  void initState() {
    if (widget?.selected != null) {
      _selected = []..addAll(widget.selected);
    }
    super.initState();
  }

  void _clearSearch() {
    if (mounted) {
      setState(() {
        _controller.clear();
        _query = '';
      });
    }
  }

  List<Lead> _selected = [];
  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentUser(context);
    final db = GetIt.instance<DatabaseService>();
    return Scaffold(
      appBar: AppBar(
        actions: <Widget>[
          IconButton(
            icon: Icon(Icons.save),
            onPressed: _selected == null || _selected.isEmpty
                ? null
                : () => Navigator.pop(context, _selected),
          ),
        ],
      ),
      body: Column(
        children: <Widget>[
          Container(
            padding: const EdgeInsets.all(8.0),
            child: TextField(
              controller: _controller,
              decoration: InputDecoration(
                  hintText: 'Search..',
                  border: InputBorder.none,
                  filled: true,
                  suffixIcon: IconButton(
                    icon: Icon(Icons.clear),
                    onPressed: _clearSearch,
                  )),
              onChanged: (val) {
                _debouncer.run(() {
                  if (mounted)
                    setState(() {
                      _query = val;
                    });
                });
              },
            ),
          ),
          Expanded(
            child: FutureBuilder<List<Lead>>(
              future: db.leadsDao.searchLeads(_user.userID, _query),
              builder: (context, snapshot) {
                if (snapshot.hasData) {
                  final _items = snapshot.data;
                  return Container(
                    child: ListView.builder(
                      itemCount: _items.length,
                      itemBuilder: (context, index) {
                        final item = _items[index];
                        final isSelected = _selected.contains(item);
                        return ListTile(
                          leading: Icon(isSelected
                              ? Icons.radio_button_checked
                              : Icons.radio_button_unchecked),
                          // selected: isSelected,
                          title: Text(item.firstName),
                          subtitle: Text(item.lastActivity),
                          onTap: () {
                            if (mounted)
                              setState(() {
                                if (isSelected) {
                                  _selected.remove(item);
                                } else {
                                  _selected.add(item);
                                }
                              });
                          },
                        );
                      },
                    ),
                  );
                }
                return Center(
                  child: CircularProgressIndicator(),
                );
              },
            ),
          ),
        ],
      ),
    );
  }
}
