import 'dart:core';

import 'package:after_layout/after_layout.dart';
import 'package:dots_indicator/dots_indicator.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:intl/intl.dart';

import '../../../data/blocs/calc/bloc.dart';
import '../../../data/utils/navigate.dart';
import 'new_calc.dart';

class CalculatorScreen extends StatefulWidget {
  @override
  _CalculatorScreenState createState() => _CalculatorScreenState();
}

class _CalculatorScreenState extends State<CalculatorScreen>
    with AfterLayoutMixin<CalculatorScreen> {
  double downPay = 0.0;
  double interestAvg = 0.0;
  final money = NumberFormat.simpleCurrency();
  List<bool> monthly;
  bool isInterest = false;
  bool ipad = false;
  OverlayEntry overlayEntry;
  List<PieChartSectionData> pieChartData = [];
  double purchaseLoan;
  double interest = 0;
  double principal = 0;
  String purpose;
  var section1 = PieChartSectionData(),
      section2 = PieChartSectionData(),
      section3 = PieChartSectionData(),
      section4 = PieChartSectionData();

  double term = 30.0;

  CalculationBloc _bloc;
  int _currentindex = 0;
  List<FlSpot> principalLine = [];
  List<FlSpot> interestLine = [];
  List<FlSpot> loanLine = [];
  List<double> principalNorm = [];
  List<double> interestNorm = [];
  List<double> loanNorm = [];

  String _interestRate, _purchasePrice, _downPayment, _loanAmount;

  double _sliceRadius = 60,
      _sliceSelected = 75,
      monthlyPay = 0,
      totalMonthPay = 0,
      pmi = 0,
      propTax = 0,
      hoa = 0;

  void _init() {
    _bloc.dispatch(LoadInit());
    _bloc.state.listen((state) {
      if (mounted) {
        if (state is ValuesLoaded) {
          _interestRate = state.fields.interestRate;
          _purchasePrice = state.fields.purchasePrice;
          _loanAmount = state.fields.loanAmount;
          _downPayment = state.fields.downPayment;
          monthlyPay = state.fields.monthlyPayment;
          totalMonthPay = state.fields.totalPay;
          pmi = state.fields.pmi;
          propTax = state.fields.propTaxCalc;
          hoa = state.fields.hoaCalc;
          purpose = state.fields.loanPurpose;
          interestAvg = state.fields.interestAvg ?? _interestRate;
          purchaseLoan = state.fields.purchaseLoan;
          downPay = state.fields.downPaymentCalc;
          term = state.fields.loanTerm / 12;
          principalLine = state.fields.principalLine;
          principalNorm = state.fields.principleNorm;
          interestLine = state.fields.interestLine;
          interestNorm = state.fields.interestNorm;
          loanLine = state.fields.loanLine;
          loanNorm = state.fields.loanNorm;
          isInterest = state.fields.isInterest;

          _updateGraph();
        }
      }
    });
  }

  void _updateGraph() {
    section1 = PieChartSectionData(
      color: Colors.red,
      value: monthlyPay,
      title: '1',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      radius: 70,
    );
    section2 = PieChartSectionData(
      color: Colors.green,
      title: '2',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      value: pmi,
      radius: 70,
    );
    section3 = PieChartSectionData(
      color: Colors.blue,
      title: '3',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      value: propTax,
      radius: 70,
    );
    section4 = PieChartSectionData(
      color: Colors.yellow,
      title: '4',
      titleStyle: TextStyle(color: Colors.white.withOpacity(0)),
      value: hoa,
      radius: 70,
    );

    setState(() {
      List<PieChartSectionData> chartItems = [
        section1,
        section2,
        section3,
        section4,
      ];
      pieChartData = chartItems;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (MediaQuery.of(context).size.shortestSide > 600) {
      ipad = true;
    } else {
      ipad = false;
    }
    if (!_loaded)
      return Scaffold(
        appBar: AppBar(),
        body: Center(child: CircularProgressIndicator()),
      );
    return Scaffold(
      appBar: _buildAppBar(context),
      body: SafeArea(
        child: BlocProvider(
          builder: (context) => _bloc,
          child: Padding(
            padding: const EdgeInsets.all(10.0),
            child: LayoutBuilder(builder: (context, constraints) {
              if (!ipad) {
                return SingleChildScrollView(
                  child: Column(
                    children: <Widget>[
                      _buildGraphPages(context),
                      DotsIndicator(
                        dotsCount: 2,
                        position: _currentindex,
                        decorator: DotsDecorator(
                          activeColor: Theme.of(context).accentColor,
                          size: const Size.square(9.0),
                          activeSize: const Size(18.0, 9.0),
                          spacing: EdgeInsets.all(5),
                          activeShape: RoundedRectangleBorder(
                              borderRadius: BorderRadius.circular(5.0)),
                        ),
                      ),
                      _buildLoanInfo(context),
                    ],
                  ),
                );
              } else {
                return Column(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    Row(
                      children: <Widget>[
                        Container(
                          width: constraints.maxWidth / 2,
                          height: 300,
                          child: _buildPieCard(),
                        ),
                        Container(
                          width: constraints.maxWidth / 2,
                          height: 300,
                          child: _buildLineCard(context),
                        )
                      ],
                    ),
                    _buildLoanInfo(context),
                    SizedBox(),
                  ],
                );
              }
            }),
          ),
        ),
      ),
    );
    //);
  }

  Row _buildLegend() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
      children: <Widget>[
        buildListTile('Payment', monthlyPay, Colors.red,
            onTap: () => showGraphValue(0)),
        buildListTile('PMI', pmi, Colors.green, onTap: () => showGraphValue(1)),
        buildListTile('Property Tax', propTax, Colors.blue,
            onTap: () => showGraphValue(2)),
        buildListTile('HOA', hoa, Colors.yellow, onTap: () => showGraphValue(3))
      ],
    );
  }

  InkWell buildListTile(String title, double value, Color color,
      {VoidCallback onTap}) {
    return InkWell(
      onTap: onTap,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Padding(
            padding: const EdgeInsets.all(1.0),
            child: Icon(FontAwesomeIcons.solidCircle, size: 20, color: color),
          ),
          Padding(
            padding: const EdgeInsets.all(10.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Text(
                  title,
                  style: TextStyle(fontSize: 16),
                ),
                Text(
                  '${money.format(value)}',
                  style: Theme.of(context).textTheme.caption,
                )
              ],
            ),
          ),
        ],
      ),
    );
  }

  AppBar _buildAppBar(BuildContext context) {
    if (MediaQuery.of(context).orientation == Orientation.portrait) {
      return AppBar(
        title: Text('Loan Calculator'),
      );
    }
    return null;
  }

  void showGraphValue(int index) {
    final List<int> slices = [0, 1, 2, 3];
    for (int slice in slices) {
      if (slice != index) {
        pieChartData[slice] =
            pieChartData[slice].copyWith(radius: _sliceRadius);
      }
    }
    setState(() {
      pieChartData[index] =
          pieChartData[index].copyWith(radius: _sliceSelected);
    });
  }

  Container _buildLoanInfo(BuildContext context) {
    return Container(
      margin: EdgeInsets.only(top: 25),
      child: Column(
        children: <Widget>[
          Text(
            'Current Loan',
            style: Theme.of(context).textTheme.subhead,
          ),
          Padding(padding: EdgeInsets.all(10)),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Text('Interest Rate:\n$_interestRate%'),
                  Padding(padding: EdgeInsets.all(8)),
                  Text(
                      'Purchase price:\n${money.format(double.tryParse(_purchasePrice) ?? 0)}'),
                  Padding(padding: EdgeInsets.all(8)),
                  Text('Down Payment:\n$_downPayment%'),
                  Padding(padding: EdgeInsets.all(8)),
                  Text('Term Length:\n${term.round()} years')
                ],
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  if (purpose == 'Purchase')
                    Text('Loan Amount:\n${money.format(purchaseLoan)}'),
                  if (purpose == 'Refinance')
                    Text(
                        'Loan Amount:\n\$${money.format(double.tryParse(_loanAmount) ?? 0)}'),
                  Padding(padding: EdgeInsets.all(8)),
                  Text('Total Interest:\n${money.format(interestNorm[9])}'),
                  Padding(padding: EdgeInsets.all(8)),
                  Text(
                      'Total Repayment:\n${money.format(principalNorm[9] + interestNorm[9])}'),
                  Padding(padding: EdgeInsets.all(8)),
                  Text('Loan Purpose:\n$purpose')
                ],
              ),
            ],
          ),
          Padding(padding: EdgeInsets.all(15)),
          InkWell(
            onTap: () {
              navigate(context, NewCalculation(), fullScreen: true);
            },
            child: Container(
              height: 60,
              width: 300,
              child: Card(
                  shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(20)),
                  elevation: 5,
                  child: Center(child: Text('Calculate New Loan'))),
            ),
          )
        ],
      ),
    );
  }

  Container _buildGraphPages(BuildContext context) {
    return Container(
      width: MediaQuery.of(context).size.width,
      height: 300,
      child: PageView(
        onPageChanged: (val) {
          setState(() {
            _currentindex = val;
          });
        },
        controller: PageController(),
        children: <Widget>[
          _buildPieCard(),
          _buildLineCard(context),
        ],
      ),
    );
  }

  Card _buildPieCard() {
    return Card(
      elevation: 5,
      child: BlocBuilder(
        bloc: _bloc,
        builder: (contex, CalculationState state) {
          if (state is ValuesLoaded) {
            return Column(
              children: <Widget>[
                _buildChart(),
                Text(
                  'Total: ${money.format(totalMonthPay)}',
                ),
                Container(
                  margin: const EdgeInsets.only(top: 20.0),
                  child: _buildLegend(),
                )
              ],
            );
          } else {
            return Center(child: Text('No Data Available'));
          }
        },
      ),
    );
  }

  Container _buildChart() {
    return Container(
      width: 200,
      height: 170,
      child: FlChart(
        chart: PieChart(
          PieChartData(
            borderData: FlBorderData(show: false),
            sectionsSpace: 0,
            centerSpaceRadius: 0,
            sections: pieChartData,
          ),
        ),
      ),
    );
  }

  Card _buildLineCard(BuildContext context) {
    return Card(
      elevation: 5,
      child: BlocBuilder(
        bloc: _bloc,
        builder: (contex, CalculationState state) {
          if (state is ValuesLoaded) {
            return Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Padding(
                  padding: const EdgeInsets.only(top: 8.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      Container(
                        decoration: BoxDecoration(color: Colors.blue),
                        height: 5,
                        width: 15,
                      ),
                      Padding(
                        padding: const EdgeInsets.only(left: 8.0, right: 8),
                        child: Text('Interest'),
                      ),
                      Container(
                        decoration: BoxDecoration(color: Colors.red),
                        height: 5,
                        width: 15,
                      ),
                      Padding(
                        padding: const EdgeInsets.only(left: 8.0, right: 8),
                        child: Text('Principal'),
                      ),
                      Container(
                        decoration: BoxDecoration(color: Colors.yellow),
                        height: 5,
                        width: 15,
                      ),
                      Padding(
                        padding: const EdgeInsets.only(left: 8.0, right: 8),
                        child: Text('Loan'),
                      ),
                    ],
                  ),
                ),
                SizedBox(height: 10),
                Center(
                  child: _buildLineChart(context),
                ),
                Container(
                    //height: ,
                    )
              ],
            );
          } else {
            return Center(child: Text('No Data Available'));
          }
        },
      ),
    );
  }

  SizedBox _buildLineChart(BuildContext context) {
    return SizedBox(
      width: 350,
      height: 250,
      child: FlChart(
        chart: LineChart(
          LineChartData(
            minX: 0,
            maxX: 10,
            minY: 0,
            maxY: 10,
            lineTouchData: LineTouchData(
              enabled: false,
              // touchTooltipData: TouchTooltipData(
              //   getTooltipItems: (spots) {
              //     //Get tooltip data
              //   }
              // )
            ),
            borderData: FlBorderData(
              border:
                  Border.all(color: Theme.of(context).textTheme.headline.color),
            ),
            lineBarsData: [
              LineChartBarData(
                spots: principalLine,
                barWidth: 2,
                belowBarData: BelowBarData(
                  show: false,
                ),
              ),
              LineChartBarData(
                spots: loanLine,
                barWidth: 2,
                colors: [Colors.yellow],
                belowBarData: BelowBarData(
                  show: false,
                ),
              ),
              LineChartBarData(
                spots: interestLine,
                barWidth: 2,
                colors: [Colors.blue],
                belowBarData: BelowBarData(
                  show: false,
                ),
              ),
            ],
            titlesData: FlTitlesData(
              leftTitles: SideTitles(
                  textStyle: TextStyle(
                      fontSize: 9,
                      color: Theme.of(context).textTheme.headline.color),
                  showTitles: true,
                  getTitles: (value) {
                    double loanP = double.tryParse(_loanAmount);
                    double interestP = interestNorm[9];
                    if (!isInterest) {
                      return '\$${(((value / 10) * loanP) / 1000).round()}k';
                    } else {
                      return '\$${(((value / 10) * interestP) / 1000).round()}k';
                    }
                  }),
              bottomTitles: SideTitles(
                showTitles: true,
                getTitles: (value) {
                  final val = ((term / 10) * value);
                  if (value == 0) {
                    return '${(interestLine[value.toInt()].x).toStringAsFixed(0)}';
                  }
                  if (val % 1 == 0) {
                    return '${val.toStringAsFixed(0)}';
                  } else {
                    return '${(term / 10) * value}';
                  }
                },
                textStyle: TextStyle(
                    fontSize: 11,
                    color: Theme.of(context).textTheme.headline.color),
              ),
            ),
          ),
        ),
      ),
    );
  }

  bool _loaded = false;
  @override
  void afterFirstLayout(BuildContext context) {
    _bloc = BlocProvider.of<CalculationBloc>(context);
    _init();
    _loaded = true;
  }
}
