import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import '../../../data/services/database/database.dart';
import '../../contacts/common/detailed_tile.dart';
import '../../contacts/common/more_button.dart';

class DetailedLoanTile extends StatelessWidget {
  DetailedLoanTile({
    Key key,
    @required loan,
    this.name,
    this.subtitle,
  })  : _loan = loan,
        super(key: key);

  final Loan _loan;
  final String name;
  final String subtitle;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: EdgeInsets.all(8),
      child: LayoutBuilder(
        builder: (context, dimens) => SizedBox(
            height: 100,
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Icon(Icons.folder, size: 45),
                Expanded(
                  child: Padding(
                    padding: EdgeInsets.only(left: 10, right: 10),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        Text(
                          name.trim().isEmpty ? 'No Name Found' : name.trim(),
                          style: Theme.of(context).textTheme.title,
                          maxLines: 1,
                        ),
                        Padding(padding: EdgeInsets.only(bottom: 6)),
                        Text(subtitle,
                            style: Theme.of(context).textTheme.subtitle,
                            maxLines: 1),
                        if (dimens.maxWidth < 450) _buildMore(),
                      ],
                    ),
                  ),
                ),
                if (dimens.maxWidth >= 450) _buildMore(),
                Column(
                  crossAxisAlignment: CrossAxisAlignment.end,
                  children: <Widget>[
                    MoreButton(
                      cell: _loan.cellPhone,
                      email: _loan.email,
                      home: _loan.homePhone,
                      office: _loan.officePhone,
                    ),
                  ],
                )
              ],
            )),
      ),
    );
  }

  Row _buildMore() {
    return Row(
      children: <Widget>[
        TileAction(
          label: _loan.loanRate.toStringAsFixed(2),
          tooltip: 'Interest Rate',
          avatar: Icon(FontAwesomeIcons.percentage),
        )
      ],
    );
  }
}
