import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:groovin_widgets/groovin_widgets.dart';
import 'package:keyboard_visibility/keyboard_visibility.dart';

import '../../../data/blocs/calc/bloc.dart';
import '../../common/keyboard_done.dart';


class NewCalculation extends StatefulWidget {
  const NewCalculation({Key key}) : super(key: key);

  @override
  _NewCalculationState createState() => _NewCalculationState();
}

class _NewCalculationState extends State<NewCalculation> {
  OverlayEntry overlayEntry;
  String purpose;
  double term;
  List<double> _terms = [10, 15, 20, 25, 30, 35, 40];
  bool monthly = false, valid = false, showOptional = false;

  CalculationBloc _bloc;

  // -- Validation -- \\
  final GlobalKey<FormState> _formKey = GlobalKey<FormState>();
  bool _autovalidate = false;

  // -- Controllers -- \\
  TextEditingController _interestRate = TextEditingController(),
      _purchasePrice = TextEditingController(),
      _downPayment = TextEditingController(),
      _loanAmount = TextEditingController(),
      _propertyTaxes = TextEditingController(),
      _homeownersInsuranse = TextEditingController(),
      _hoaDues = TextEditingController();
  List<TextEditingController> optionalList;
  List<String> fields;
  List<String> values;

  @override
  void initState() {
    optionalList = [_propertyTaxes, _homeownersInsuranse, _hoaDues];
    KeyboardVisibilityNotification().addNewListener(onShow: () {
      showOverlay(context);
    }, onHide: () {
      removeOverlay();
    });
    super.initState();
  }

  showOverlay(BuildContext context) {
    if (overlayEntry != null) return;
    OverlayState overlayState = Overlay.of(context);
    overlayEntry = OverlayEntry(builder: (context) {
      return Positioned(
          bottom: MediaQuery.of(context).viewInsets.bottom,
          right: 0.0,
          left: 0.0,
          child: InputDoneView());
    });

    overlayState.insert(overlayEntry);
  }

  removeOverlay() {
    if (overlayEntry != null) {
      overlayEntry.remove();
      overlayEntry = null;
    }
  }

  void _calculate() {
    final FormState form = _formKey.currentState;
    if (!form.validate()) {
      _autovalidate = true;
    } else {
      fields = [
        'Interest Rate',
        'Purchase Price',
        'Loan Amount',
        'Down Payment',
        'Property Taxes',
        'Homeowners Insurance',
        'HOA Dues',
        'Term'
      ];
      values = [
        _interestRate.text,
        _purchasePrice.text,
        _loanAmount.text,
        _downPayment.text,
        _propertyTaxes.text,
        _homeownersInsuranse.text,
        _hoaDues.text,
        term.round().toString()
      ];
      _bloc.dispatch(
          UpdateCalculation(values, fields, monthly, purpose, reCalc: true));
      Navigator.of(context).pop();
    }
  }

  @override
  Widget build(BuildContext context) {
    _bloc = BlocProvider.of<CalculationBloc>(context);
    return Scaffold(
      appBar: AppBar(
        title: Text('New Loan'),
      ),
      body: SafeArea(
        child: Form(
          key: _formKey,
          autovalidate: _autovalidate,
          child: SingleChildScrollView(
            child: _buildInfo(),
          ),
        ),
      ),
    );
  }

  Padding _buildOptionals() {
    return Padding(
      padding: EdgeInsets.all(0),
      child: Column(
        children: <Widget>[
          Container(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Column(
                children: <Widget>[
                  SwitchListTile.adaptive(
                    title: Text('Monthly:'),
                    value: monthly,
                    onChanged: (val) {
                      setState(() {
                        monthly = val;
                        if (monthly == true) {
                          for (var val in optionalList) {
                            if (val.text.isNotEmpty) {
                              val.text = (double.tryParse(val.text) / 12)
                                  .toStringAsFixed(2);
                            }
                          }
                        } else {
                          for (var val in optionalList) {
                            if (val.text.isNotEmpty) {
                              val.text = (double.tryParse(val.text) * 12)
                                  .toStringAsFixed(2);
                            }
                          }
                        }
                      });
                    },
                  ),
                  _buildTile(
                      label: 'Property Tax',
                      trailing: Icon(FontAwesomeIcons.percentage, size: 20),
                      controller: _propertyTaxes,
                      isOptional: true,
                      isPercent: true),
                  _buildTile(
                      label: 'Homeowners Insurance',
                      controller: _homeownersInsuranse,
                      isOptional: true),
                  _buildTile(
                      label: 'HOA Fees',
                      controller: _hoaDues,
                      isOptional: true),
                  Padding(padding: EdgeInsets.only(bottom: 20)),
                ],
              ),
            ),
          )
        ],
      ),
    );
  }

  Padding _buildInfo() {
    return Padding(
      padding: EdgeInsets.all(10),
      child: Column(
        children: <Widget>[
          Container(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Column(
                children: <Widget>[
                  Row(
                    children: <Widget>[
                      Expanded(
                        child: OutlineDropdownButtonFormField(
                            hint: Text("Loan Type"),
                            value: purpose,
                            isDense: false,
                            validator: (val) => _validateDropDown(
                                  val,
                                  label: "Purpose Required",
                                ),
                            items: [
                              DropdownMenuItem<String>(
                                value: 'Purchase',
                                child: Text('Purchase'),
                              ),
                              DropdownMenuItem<String>(
                                value: 'Refinance',
                                child: Text('Refinance'),
                              )
                            ],
                            onChanged: (val) {
                              setState(() {
                                purpose = val;
                              });
                            }),
                      ),
                      Padding(padding: EdgeInsets.all(5)),
                      Expanded(
                        child: OutlineDropdownButtonFormField(
                          hint: Text('Loan Term'),
                          value: term,
                          validator: (val) => _validateDropDown(
                            val,
                            label: "Term Required",
                          ),
                          items: _terms
                              .map((item) => DropdownMenuItem<double>(
                                    value: item,
                                    child: Text(
                                        '${item.toStringAsFixed(0)} years'),
                                  ))
                              .toList(),
                          onChanged: (val) {
                            setState(() {
                              term = val;
                            });
                          },
                        ),
                      )
                    ],
                  ),
                  Padding(padding: EdgeInsets.all(10)),
                  _buildTile(
                      label: "interest Rate",
                      trailing: Icon(FontAwesomeIcons.percentage, size: 20),
                      controller: _interestRate,
                      isOptional: false,
                      isPercent: true),
                  _buildTile(
                      label: "Down Payment",
                      trailing: Icon(
                        FontAwesomeIcons.percentage,
                        size: 20,
                      ),
                      controller: _downPayment,
                      isOptional: false,
                      isPercent: true),
                  _buildTile(
                      label: "Purchase Price",
                      controller: _purchasePrice,
                      isOptional: false),
                  if (purpose == 'Refinance')
                    _buildTile(
                        label: "Loan Amount",
                        numeric: true,
                        controller: _loanAmount,
                        isOptional: false),
                  Theme(
                    data: Theme.of(context)
                        .copyWith(dividerColor: Colors.white.withOpacity(0)),
                    child: ExpansionTile(
                      title: Text(
                        'Show Optional Entries',
                        style: Theme.of(context).textTheme.button,
                      ),
                      children: <Widget>[_buildOptionals()],
                    ),
                  ),
                  Padding(padding: EdgeInsets.only(bottom: 20)),
                  InkWell(
                    onTap: () {
                      _calculate();
                    },
                    child: Container(
                      height: 60,
                      width: 300,
                      child: Card(
                          shape: RoundedRectangleBorder(
                              borderRadius: BorderRadius.circular(20)),
                          elevation: 5,
                          child: Center(child: Text('Calculate'))),
                    ),
                  )
                ],
              ),
            ),
          )
        ],
      ),
    );
  }

  Widget _buildTile(
      {String label = "",
      bool required = true,
      bool numeric = true,
      Widget trailing,
      Widget leading,
      ValueChanged<String> onSaved,
      ValueChanged<String> onChanged,
      TextEditingController controller,
      bool isOptional,
      bool isPercent = false}) {
    return Padding(
      padding: EdgeInsets.only(bottom: 5),
      child: Row(
        children: <Widget>[
          Expanded(
            child: TextFormField(
              decoration: InputDecoration(
                labelText: label,
                suffixIcon: controller.text.isNotEmpty
                    ? Padding(
                        padding: EdgeInsetsDirectional.only(start: 12),
                        child: IconButton(
                          iconSize: 16,
                          icon: Icon(
                            Icons.clear,
                            color: Colors.grey,
                          ),
                          onPressed: () {
                            setState(() {
                              WidgetsBinding.instance.addPostFrameCallback(
                                  (_) => controller.clear());
                            });
                          },
                        ),
                      )
                    : null,
              ),
              textAlign: TextAlign.left,
              keyboardType: TextInputType.numberWithOptions(
                decimal: true,
              ),
              textInputAction: TextInputAction.done,
              controller: controller,
              validator: required
                  ? (val) => _validateNumber(val,
                      label: "$label Required",
                      isOptional: isOptional,
                      isPercent: isPercent)
                  : null,
              onSaved: onSaved,
              onChanged: (val) {
                setState(() {});
              },
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(left: 8.0),
            child: trailing,
          )
        ],
      ),
    );
  }

  String _validateDropDown(dynamic value, {String label}) {
    if (value == null) {
      valid = false;
      return label ?? 'Required';
    }
    return null;
  }

  String _validateNumber(String value,
      {String label, bool isOptional, bool isDouble = true, isPercent}) {
    if (value.isEmpty && isOptional == false) {
      valid = false;
      return label ?? 'Number is required.';
    }
    final RegExp nameExp = RegExp(r'^[A-Za-z ]+$');
    if (nameExp.hasMatch(value)) {
      valid = false;
      return 'Please enter only numerial characters.';
    }
    if (isDouble) {
      if (isOptional == false || (isOptional && value.isNotEmpty)) {
        double _number = double.tryParse(value);
        if (_number == null) {
          valid = false;
          return 'Please enter a valid number';
        }
      }
    }
    if (isPercent == true) {
      double _number = double.tryParse(value);
      if (_number != null) {
        if (_number > 100) {
          return 'Please enter a valid percentage';
        }
      }
    }
    return null;
  }
}
