import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:intl/intl.dart';
import 'package:unify_api/unify.dart';

import '../../data/blocs/bloc.dart';
import 'library/start_campaign.dart';

class MarketingScreen extends StatelessWidget {
  DateFormat formatter = new DateFormat('MM-dd-yyyy');
  static const String routeName = '/marketing';
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        child: BlocBuilder<AuthBloc, AuthState>(
          builder: (context, state) {
            if (state is LoggedInState) {
              return Container(
                child: FutureBuilder<List<MarketingCampaign>>(
                  future: state.info.client.listMarketingCampaigns(),
                  builder: (context, snapshot) {
                    final _list = snapshot.data;
                    if (snapshot.hasData) {
                      if (_list.isEmpty) {
                        return Center(
                          child: Text('No Active Campaigns'),
                        );
                      }
                      return ListView.separated(
                        separatorBuilder: (context, index) => Divider(), 
                        itemCount: _list.length,
                        itemBuilder: (context, index) {
                          final item = _list[index];
                          return ExpansionTile(
                            title: ListTile(
                              title: Text(item?.name ?? 'No Name Found'),
                              subtitle: Text(item?.status ?? 'Unknown'),
                              //onTap: () => navigate,
                            ),
                            children: <Widget>[
                              Padding(
                                padding: const EdgeInsets.all(10.0),
                                child: ListTile(
                                  dense: true,
                                  title: Text('Campaign Type:'),
                                  trailing: Text(item.type),
                                ),
                              ),
                              Padding(
                                padding: const EdgeInsets.all(10.0),
                                child: ListTile(
                                  dense: true,
                                  title: Text('Campaign Status:'),
                                  trailing: Text(item.status),
                                ),
                              ),
                              Padding(
                                padding: const EdgeInsets.all(10.0),
                                child: ListTile(
                                  dense: true,
                                  title: Text('End Date:'),
                                  trailing: Text(formatter.format(item.endDate)),
                                ),
                              )
                            ],
                          );
                        },
                      );
                    }
                    return Center(
                      child: CircularProgressIndicator(),
                    );
                  },
                ),
              );
            }
            return Container();
          },
        ),
      ),
      floatingActionButton: FloatingActionButton(
        child: Icon(Icons.add),
        heroTag: 'start_campaign',
        onPressed: () =>
            Navigator.pushNamed(context, StartCampaignScreen.routeName),
      ),
    );
  }
}
