import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import 'account/screen.dart';
import 'auth/check.dart';
import 'auth/screen.dart';
import 'contacts/check.dart';
import 'contacts/screen.dart';
import 'leads/check.dart';
import 'leads/screen.dart';
import 'loans/check.dart';
import 'loans/screen.dart';
import 'marketing/library/start_campaign.dart';
import 'marketing/screen.dart';
import 'video/screen.dart';

class Router {
  static String initialRoute = AuthCheck.routeName;
  static Route<dynamic> onUnknownRoute(
    BuildContext context, {
    @required RouteSettings settings,
  }) {
    final route = settings.name;
    if (route.contains('/contacts')) {
      final id = route.replaceAll('/contacts/', '');
      return MaterialPageRoute(
        builder: (_) => ContactsCheck(id: id),
        settings: settings,
      );
    }
    if (route.contains('/leads')) {
      final id = route.replaceAll('/leads/', '');
      return MaterialPageRoute(
        builder: (_) => LeadsCheck(id: id),
        settings: settings,
      );
    }
    if (route.contains('/loans')) {
      final id = route.replaceAll('/loans/', '');
      return MaterialPageRoute(
        builder: (_) => LoansCheck(id: id),
        settings: settings,
      );
    }
    return MaterialPageRoute(
      builder: (_) => UnknownRouteScreen(settings: settings),
      settings: settings,
    );
  }

  static Map<String, WidgetBuilder> routes(BuildContext context) {
    return {
      LoginScreen.addRoute: (_) => LoginScreen(addAccount: true),
      LoginScreen.loginRoute: (_) => LoginScreen(addAccount: false),
      AuthCheck.routeName: (_) => AuthCheck(),
      MarketingScreen.routeName: (_) => MarketingScreen(),
      StartCampaignScreen.routeName: (_) => StartCampaignScreen(),
      LoansScreen.routeName: (_) => LoansScreen(),
      AccountScreen.route: (_) => AccountScreen(),
      ContactsScreen.routeName: (_) => ContactsScreen(),
      LeadsScreen.routeName: (_) => LeadsScreen(),
      VideosScreen.routeName: (_) => VideosScreen(),
    };
  }
}

class UnknownRouteScreen extends StatelessWidget {
  final RouteSettings settings;

  const UnknownRouteScreen({
    Key key,
    @required this.settings,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(),
      body: Center(child: Text('No route defined for ${settings.name}')),
    );
  }
}
