part of global_search;

class _SearchBody extends StatefulWidget {
  @override
  __SearchBodyState createState() => __SearchBodyState();
}

class __SearchBodyState extends State<_SearchBody> {
  List<String> colors = [];

  @override
  Widget build(BuildContext context) {
    return SettingBuilder(
      builder: (context, settings, bloc) =>
          BlocBuilder<SearchBloc, SearchState>(
        bloc: BlocProvider.of<SearchBloc>(context),
        builder: (context, state) {
          if (state is SearchStateEmpty) {
            return Center(child: Text('Please enter a term to begin'));
          }
          if (state is SearchStateLoading) {
            return Center(child: CircularProgressIndicator());
          }
          if (state is SearchStateError) {
            return Center(child: Text(state.error));
          }
          if (state is SearchStateSuccess) {
            return Container(
              child: state.items.isEmpty
                  ? Center(child: Text('No Results'))
                  : Container(
                      child: _SearchResults(
                      items: state.items,
                      contactsColor: settings.contactColor,
                      loansColor: settings.loanColor,
                      leadColor: settings.leadColor,
                      alertColor: settings.alertColor,
                      videoColor: settings.videoColor,
                    )),
            );
          }
          return Container(child: Center(child: Text('Error')));
        },
      ),
    );
  }
}

class _SearchResults extends StatelessWidget {
  const _SearchResults(
      {Key key,
      this.items,
      @required this.contactsColor,
      @required this.loansColor,
      @required this.leadColor,
      @required this.alertColor,
      @required this.videoColor})
      : super(key: key);

  final List<SearchResultItem> items;
  final int contactsColor, loansColor, alertColor, leadColor, videoColor;

  BoxDecoration _buildBorder(Color color) {
    return BoxDecoration(
        border: Border(
      left: BorderSide(
        color: color,
        width: 5.0,
      ),
    ));
  }

  @override
  Widget build(BuildContext context) {
    final _user = AuthBloc.currentInfo(context);
    int favId = 0;
    return AnimationLimiter(
      child: ListView.separated(
        separatorBuilder: (context, index) => Divider(height: 1),
        itemCount: items.length,
        itemBuilder: (context, index) {
          final item = items[index];
          switch (item.type) {
            case SearchResultType.loans:
              return AnimationConfiguration.staggeredList(
                position: index,
                duration: const Duration(milliseconds: 100),
                child: SlideAnimation(
                  verticalOffset: 50,
                  child: FadeInAnimation(
                    child: StreamBuilder<List<Favorite>>(
                        stream: GetIt.instance<DatabaseService>()
                            .favoritesDao
                            .watchFavoriteByTypeId(
                                _user.id, Loan.fromJson(item.data).loanId),
                        builder: (context, snapshot) {
                          if (snapshot.hasData && snapshot.data.isNotEmpty) {
                            favId = snapshot.data.first.favoriteId;
                          }
                          return LoanRowTile(
                            favorite:
                                snapshot.hasData && snapshot.data.isNotEmpty,
                            favId: favId,
                            border: _buildBorder(Color(loansColor)),
                            type: ViewType.dense,
                            loan: Loan.fromJson(item.data),
                            iconColor: Color(loansColor),
                          );
                        }),
                  ),
                ),
              );
            case SearchResultType.contacts:
              return AnimationConfiguration.staggeredList(
                  position: index,
                  duration: const Duration(milliseconds: 100),
                  child: SlideAnimation(
                    verticalOffset: 50,
                    child: FadeInAnimation(
                      child: StreamBuilder<List<Favorite>>(
                          stream: GetIt.instance<DatabaseService>()
                              .favoritesDao
                              .watchFavoriteByTypeId(_user.id,
                                  Contact.fromJson(item.data).contactId),
                          builder: (context, snapshot) {
                            if (snapshot.hasData && snapshot.data.isNotEmpty) {
                              favId = snapshot.data.first.favoriteId;
                            }
                            return ContactRowTile(
                              favId: favId,
                              favorite:
                                  snapshot.hasData && snapshot.data.isNotEmpty,
                              border: _buildBorder(Color(contactsColor)),
                              type: ViewType.dense,
                              contact: Contact.fromJson(item.data),
                              iconColor: Color(contactsColor),
                            );
                          }),
                    ),
                  ));
            case SearchResultType.leads:
              return AnimationConfiguration.staggeredList(
                  position: index,
                  duration: const Duration(milliseconds: 100),
                  child: SlideAnimation(
                    verticalOffset: 50,
                    child: FadeInAnimation(
                        child: StreamBuilder<List<Favorite>>(
                            stream: GetIt.instance<DatabaseService>()
                                .favoritesDao
                                .watchFavoriteByTypeId(
                                    _user.id, Lead.fromJson(item.data).leadId),
                            builder: (context, snapshot) {
                              if (snapshot.hasData &&
                                  snapshot.data.isNotEmpty) {
                                favId = snapshot.data.first.favoriteId;
                              }
                              return LeadRowTile(
                                favId: favId,
                                favorite: snapshot.hasData &&
                                    snapshot.data.isNotEmpty,
                                border: _buildBorder(Color(leadColor)),
                                type: ViewType.dense,
                                lead: Lead.fromJson(item.data),
                                iconColor: Color(leadColor),
                              );
                            })),
                  ));
            case SearchResultType.alerts:
              return AnimationConfiguration.staggeredList(
                  position: index,
                  duration: const Duration(milliseconds: 100),
                  child: SlideAnimation(
                    verticalOffset: 50,
                    child: FadeInAnimation(
                        child: AlertRowTile(
                      border: _buildBorder(Color(alertColor)),
                      type: ViewType.dense,
                      alert: Dashboard.fromJson(item.data),
                      iconColor: Color(alertColor),
                    )),
                  ));
            case SearchResultType.video:
              return AnimationConfiguration.staggeredList(
                  position: index,
                  duration: const Duration(milliseconds: 100),
                  child: SlideAnimation(
                    verticalOffset: 50,
                    child: FadeInAnimation(
                        child: VideoRowTile(
                      video: Video.fromJson(item.data),
                      type: ViewType.dense,
                      border: _buildBorder(Color(videoColor)),
                      iconColor: Color(videoColor),
                    )),
                  ));
            default:
              return Container();
          }
        },
      ),
    );
  }
}
