import 'package:example_db/data/blocs/auth/auth_bloc.dart';
import 'package:flutter/material.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:get_it/get_it.dart';

import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import '../../../data/utils/navigate.dart';
import '../../../plugins/video_player/video_player.dart';
import 'detailed_video_tile.dart';

class VideoRowTile extends StatelessWidget {
  VideoRowTile(
      {Key key,
      @required Video video,
      @required ViewType type,
      this.border,
      this.iconColor,
      this.onTap,
      this.onLongPress,
      this.archived = false,
      this.shared = false})
      : _video = video,
        _type = type,
        super(key: key);

  final BoxDecoration border;
  final Color iconColor;
  final VoidCallback onTap, onLongPress;

  final ViewType _type;
  final Video _video;

  final bool archived;
  final bool shared;

  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentUser(context);
    switch (_type) {
      case ViewType.detailed:
        return Slidable(
          actionPane: SlidableDrawerActionPane(),
          actionExtentRatio: 0.25,
          child: ExpansionTile(
            title: ListTile(
              onTap: () {
                navigate(context, DetailedVideoPage(video: _video),
                    fullScreen: true);
              },
              leading: Icon(FontAwesomeIcons.video),
              title: Text('${_video.name}'),
              subtitle: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Container(
                    width: 175,
                    child: Text(
                      _video.description,
                      maxLines: 1,
                      softWrap: true,
                      overflow: TextOverflow.ellipsis,
                    ),
                  ), 
                  Text('${_video.views} views', maxLines: 1,)
                ],
              ),
            ),
            children: <Widget>[
              Container(
                child: VideoPlayerView(
                  src: _video.download,
                  width: _video.width,
                  height: _video.height,
                ),
              )
            ],
          ),
          actions: <Widget>[
            if (!archived)
              IconSlideAction(
                caption: 'Archive',
                color: Colors.red,
                icon: Icons.archive,
                onTap: () {
                  GetIt.instance<DatabaseService>()
                      .videoFolderDao
                      .archivedVid(user.userID, _video.videoId, true);
                },
              ),
            if (archived)
              IconSlideAction(
                caption: 'Un Archive',
                color: Colors.red,
                icon: Icons.unarchive,
                onTap: () {
                  GetIt.instance<DatabaseService>()
                      .videoFolderDao
                      .removeItem(user.userID, _video.videoId);
                },
              ),
          ],
          secondaryActions: <Widget>[
            IconSlideAction(
              caption: 'Share',
              color: Colors.blueAccent,
              icon: Icons.share,
              onTap: () {},
            )
          ],
        );
        break;
      case ViewType.dense:
        return Slidable(
          actionPane: SlidableDrawerActionPane(),
          actionExtentRatio: 0.25,
          child: InkWell(
            child: Container(
              decoration: border,
              child: ExpansionTile(
                title: ListTile(
                  onTap: () {
                    navigate<bool>(context, DetailedVideoPage(video: _video),
                        fullScreen: true);
                  },
                  title: Text('${_video.name}'),
                  subtitle: Text('${_video.description}',
                      softWrap: true,
                      overflow: TextOverflow.ellipsis,
                      style: Theme.of(context)
                          .textTheme
                          .caption
                          .copyWith(fontSize: 14)),
                  leading: Icon(
                    FontAwesomeIcons.video,
                    color: iconColor,
                  ),
                ),
                children: <Widget>[
                  Container(
                    child: VideoPlayerView(
                      src: _video.download,
                      width: _video.width,
                      height: _video.height,
                    ),
                  )
                ],
              ),
            ),
          ),
          secondaryActions: <Widget>[
            IconSlideAction(
              caption: 'Archive',
              color: Colors.green,
              icon: Icons.archive,
              onTap: () {
                GetIt.instance<DatabaseService>()
                    .videoFolderDao
                    .archivedVid(user.userID, _video.videoId, true);
              },
            )
          ],
        );
        break;
    }
  }
}
