part of unify;

class PhoneNumber implements UnifyObject {
  PhoneNumber(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get label => json["label"];
  String get areaCode => json["area_code"];
  String get prefix => json["prefix"];
  String get number => json["number"];
  String get ext => json["ext"];

  // int get timestamp => json["timestamp"];
  // DateTime get timestampTime =>
  //     new DateTime.fromMillisecondsSinceEpoch(timestamp);

  /// Get the raw Phone Number eg. 387338763
  String raw() {
    if (areaCode == null && prefix == null && number == null) {
      return "";
    }
    try {
      if (ext != null) {
        return "$areaCode$prefix$number,$ext".toString();
      }
      return "$areaCode$prefix$number".toString();
    } catch (e) {
      print(e);
      return "";
    }
  }

  /// Get a Phone Number from a 10 digit string (after symbols removed)
  Map<String, dynamic> phoneNumberFromString(String value, {String name}) {
    Map<String, dynamic> _json = Map();
    _json["label"] = name ?? "phone";
    var _number = replaceCommon(value);
    if (_number.length >= 10) {
      _json["area_code"] = _number.substring(0, 3);
      _json["prefix"] = _number.substring(3, 6);
      _json["number"] = _number.substring(6, 10);
      if (_number.length > 10) {
        _json["ext"] = value.substring(10, _number.length);
      }
    }
    return _json;
  }

  /// Remove Common Symbols in Phone Numbers
  String replaceCommon(String value) {
    var _number = value
        .replaceAll("+", "")
        .replaceAll("-", "")
        .replaceAll("(", "")
        .replaceAll(")", "")
        .replaceAll(" ", "")
        .replaceAll(",", "")
        .trim();
    return _number;
  }

  @override
  String toString() {
    if (raw().isEmpty) return "";
    if (ext != null) {
      return "($areaCode) $prefix-$number :$ext".toString();
    }
    return "($areaCode) $prefix-$number".toString();
  }
}
