part of unify;

class CoreLead implements UnifyObject {
  CoreLead(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get id => json["Lead_Tracker_Lead_ID"];

  String get contactId => json["Contact_ID"];

  String get fullName => "$firstName $lastName".trim();

  String get displayName => "$lastName, $firstName";

  String get firstName => json["Borrower_First_Name"];

  String get lastName => json["Borrower_Last_Name"];

  PhoneNumber get primaryPhone => PhoneNumber(client, {
        "label": "primary",
        "area_code": json["Borrower_Phone1_Area_Code"],
        "prefix": json["Borrower_Phone1_Prefix"],
        "number": json["Borrower_Phone1_Number"],
      });

  PhoneNumber get secondaryPhone => PhoneNumber(client, {
        "label": "secondary",
        "area_code": json["Borrower_Phone2_Area_Code"],
        "prefix": json["Borrower_Phone2_Prefix"],
        "number": json["Borrower_Phone2_Number"],
      });

  String get email => json["Borrower_Email_Address"];

  PostalAddress get mailingAddress => PostalAddress(client, {
        "label": "mailing",
        "street": json["Mailing_Address1"],
        "apartment": json["Mailing_Address2"],
        "city": json["Mailing_City"],
        "state": json["Mailing_State"],
        "zip": json["Mailing_Zip"],
      });

  String get loanNumber => json["Loan_Number"];

  String get loanPurpose => json["Loan_Purpose"];

  String get cofullName => "$firstName $lastName".trim();

  String get codisplayName => "$lastName, $firstName";

  String get cofirstName => json["CoBorrower_First_Name"];

  String get colastName => json["CoBorrower_Last_Name"];

  String get leadSource => json["Lead_Source"];

  String get leadSourceType => json["Lead_Source_Type"];

  String get leadSourceName => json["Lead_Source_Name"];

  PhoneNumber get coprimaryPhone => PhoneNumber(client, {
        "label": "primary",
        "area_code": json["CoBorrower_Phone1_Area_Code"],
        "prefix": json["CoBorrower_Phone1_Prefix"],
        "number": json["CoBorrower_Phone1_Number"],
      });

  PhoneNumber get cosecondaryPhone => PhoneNumber(client, {
        "label": "secondary",
        "area_code": json["CoBorrower_Phone2_Area_Code"],
        "prefix": json["CoBorrower_Phone2_Prefix"],
        "number": json["CoBorrower_Phone2_Number"],
      });

  PhoneNumber get leadSourceCellPhone => PhoneNumber(client, {
        "label": "cell",
        "area_code": json["Lead_Source_Cell_Phone_Area_Code"],
        "prefix": json["Lead_Source_Cell_Phone_Prefix"],
        "number": json["Lead_Source_Cell_Phone_Number"],
      });

  PhoneNumber get leadSourceOfficePhone => PhoneNumber(client, {
        "label": "office",
        "area_code": json["Lead_Source_Office_Phone_Area_Code"],
        "prefix": json["Lead_Source_Office_Phone_Prefix"],
        "number": json["Lead_Source_Office_Phone_Number"],
      });

  String get leadSourceEmail => json["Lead_Source_Email_Address"];

  PhoneNumber get salesAgentCellPhone => PhoneNumber(client, {
        "label": "cell",
        "area_code": json["Sales_Agent_Cell_Phone_Area_Code"],
        "prefix": json["Sales_Agent_Cell_Phone_Prefix"],
        "number": json["Sales_Agent_Cell_Phone_Number"],
      });

  PhoneNumber get salesAgentOfficePhone => PhoneNumber(client, {
        "label": "office",
        "area_code": json["Sales_Agent_Office_Phone_Area_Code"],
        "prefix": json["Sales_Agent_ffice_Phone_Prefix"],
        "number": json["Sales_Agent_Office_Phone_Number"],
      });

  String get salesAgentEmail => json["Sales_Agent_Email_Address"];

  String get coemail => json["CoBorrower_Email_Address"];

  bool matchesSearch(String search) {
    if (search == null || search.isEmpty) {
      return true;
    }

    search = search.toLowerCase();

    if (firstName.toLowerCase().contains(search)) {
      return true;
    }
    if (lastName.toLowerCase().contains(search)) {
      return true;
    }
    if (primaryPhone.raw().toLowerCase().contains(search)) {
      return true;
    }
    if (secondaryPhone.raw().toLowerCase().contains(search)) {
      return true;
    }
    if (email.toLowerCase().contains(search)) {
      return true;
    }
    return false;
  }

  // -- Actions --
  Future<CoreLeadDetails> getDetails() {
    return client.getCoreLeadDetails(id);
  }

  Future<List<Task>> listTasks({DateTime date}) {
    return client.listTasksForCoreLead(id, date: date);
  }
}
