part of unify;

class DashboardAlert
    implements UnifyObject, QueryObject, SortableObject<DashboardAlert> {
  DashboardAlert(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get id => json["alert_id"];
  String get contactId => json["contact_id"];
  String get loanId => json["loan_id"];
  String get fullName => "$firstName $lastName".trim();
  String get displayName => "$lastName, $firstName";
  String get firstName => json["first_name"];
  String get lastName => json["last_name"];
  String get email => json["email"];
  String get cellPhone => json["cell_phone"];
  String get homePhone => json["home_phone"];
  String get officePhone => json["office_phone"];
  num get interestRate => json["interest_rate"];
  String get otherInfo => json["other_info"];
  String get loanNumber => json["loan_number"];
  String get alertType => json["alert_type"];
  String get alertPriority => json["alert_priority"];
  DateTime get dateCreated => DateTime.tryParse(json["date_created"] as String);
  DateTime get dateDeleted => DateTime.tryParse(json["date_deleted"] as String);
  bool get deleted => json["deleted"];

  // -- Actions --
  Future<ContactDetails> getContactDetails() {
    return client.getContactDetails(contactId);
  }

  Future<LoanDetails> getLoanDetails() {
    return client.getLoanDetails(loanId);
  }

  Future delete() {
    return client.deleteDashboardAlert(id);
  }

  @override
  List<String> sortFields = [
    "first_name",
    "last_name",
    "cell_phone",
    "office_phone",
    "home_phone",
    "date_created",
    "date_modified",
    "email",
    "alert_type",
    "alert_priority",
    "loan_number",
    "interest_rate",
  ];

  @override
  int compareTo(DashboardAlert object, String sortField, bool sortAscending) {
    int response = 0;
    DashboardAlert objectA = sortAscending ? this : object;
    DashboardAlert objectB = sortAscending ? object : this;

    switch (sortField) {
      case "first_name":
        response = objectA.firstName.compareTo(objectB.firstName);
        break;
      case "last_name":
        response = objectA.lastName.compareTo(objectB.lastName);
        break;
      case "cell_phone":
        response = objectA.cellPhone.compareTo(objectB.cellPhone);
        break;
      case "office_phone":
        response = objectA.officePhone.compareTo(objectB.officePhone);
        break;
      case "home_phone":
        response = objectA.homePhone.compareTo(objectB.homePhone);
        break;
      case "email":
        response = objectA.email.compareTo(objectB.email);
        break;
      case "date_created":
        response = objectA.dateCreated.compareTo(objectB.dateCreated);
        break;
      case "alert_type":
        response = objectA.alertType.compareTo(objectB.alertType);
        break;
      case "alert_priority":
        response = objectA.alertPriority.compareTo(objectB.alertPriority);
        break;
      case "loan_number":
        response = objectA.loanNumber.compareTo(objectB.loanNumber);
        break;
      case "interest_rate":
        response = objectA.interestRate.compareTo(objectB.interestRate);
        break;
    }

    if (response == 0) {
      return objectA.firstName.compareTo(objectB.firstName);
    } else {
      return response;
    }
  }

  @override
  bool matchesSearch(String search) {
    if (search == null || search.isEmpty) {
      return true;
    }

    search = search.toLowerCase();

    if (firstName.toLowerCase().contains(search)) {
      return true;
    }
    if (lastName.toLowerCase().contains(search)) {
      return true;
    }
    if (cellPhone.toLowerCase().contains(search)) {
      return true;
    }
    if (homePhone.toLowerCase().contains(search)) {
      return true;
    }
    if (officePhone.toLowerCase().contains(search)) {
      return true;
    }
    if (email.toLowerCase().contains(search)) {
      return true;
    }
    if (alertType.toLowerCase().contains(search)) {
      return true;
    }
    if (alertPriority.toLowerCase().contains(search)) {
      return true;
    }
    if (dateCreated.toString().toLowerCase().contains(search)) {
      return true;
    }
    if (loanNumber.toString().toLowerCase().contains(search)) {
      return true;
    }
    if (interestRate.toString().toLowerCase().contains(search)) {
      return true;
    }
    return false;
  }
}
