part of unify;

class DashboardPartners
    implements UnifyObject, QueryObject, SortableObject<DashboardPartners> {
  DashboardPartners(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get id => json["Dashboard_Alert_ID"];
  String get contactId => json["Contact_ID"];
  String get loanId => json["Loan_ID"];
  String get userId => json["User_ID"];
  String get branchId => json["Branch_ID"];
  String get marketingPartnerId => json["Marketing_Partner_ID"];
  String get company => json["Company_Name"];
  String get fullName => "$firstName $lastName".trim();
  String get displayName => "$lastName, $firstName";
  String get firstName => json["First_Name"];
  String get lastName => json["Last_Name"];
  String get email => json["Email_Address"];
  String get cellPhone => json["Cell_Phone"];
  String get homePhone => json["Home_Phone"];
  String get officePhone => json["Office_Phone"];
  String get otherInfo => json["Other_Information"];
  String get loanNumber => json["Loan_Number"];
  String get alertType => json["Alert_Type"];
  String get alertPriority => json["Alert_Priority"];
  DateTime get dateCreated => DateTime.tryParse(json["Date_Created"] as String);
  DateTime get dateDeleted => DateTime.tryParse(json["Date_Deleted"] as String);
  bool get deleted => json["Deleted"];

  // -- Actions --
  Future<ContactDetails> getContactDetails() {
    return client.getContactDetails(contactId);
  }

  Future<LoanDetails> getLoanDetails() {
    return client.getLoanDetails(loanId);
  }

  Future delete() {
    return client.deleteDashboardAlert(id);
  }

  @override
  List<String> sortFields = [
    "first_name",
    "last_name",
    "cell_phone",
    "office_phone",
    "home_phone",
    "date_created",
    "date_modified",
    "email",
    "alert_type",
    "alert_priority",
    "loan_number",
  ];

  @override
  int compareTo(
      DashboardPartners object, String sortField, bool sortAscending) {
    int response = 0;
    DashboardPartners objectA = sortAscending ? this : object;
    DashboardPartners objectB = sortAscending ? object : this;

    switch (sortField) {
      case "first_name":
        response = objectA.firstName.compareTo(objectB.firstName);
        break;
      case "last_name":
        response = objectA.lastName.compareTo(objectB.lastName);
        break;
      case "cell_phone":
        response = objectA.cellPhone.compareTo(objectB.cellPhone);
        break;
      case "office_phone":
        response = objectA.officePhone.compareTo(objectB.officePhone);
        break;
      case "home_phone":
        response = objectA.homePhone.compareTo(objectB.homePhone);
        break;
      case "email":
        response = objectA.email.compareTo(objectB.email);
        break;
      case "date_created":
        response = objectA.dateCreated.compareTo(objectB.dateCreated);
        break;
      case "alert_type":
        response = objectA.alertType.compareTo(objectB.alertType);
        break;
      case "alert_priority":
        response = objectA.alertPriority.compareTo(objectB.alertPriority);
        break;
      case "loan_number":
        response = objectA.loanNumber.compareTo(objectB.loanNumber);
        break;
    }

    if (response == 0) {
      return objectA.firstName.compareTo(objectB.firstName);
    } else {
      return response;
    }
  }

  @override
  bool matchesSearch(String search) {
    if (search == null || search.isEmpty) {
      return true;
    }

    search = search.toLowerCase();

    if (firstName.toLowerCase().contains(search)) {
      return true;
    }
    if (lastName.toLowerCase().contains(search)) {
      return true;
    }
    if (cellPhone.toLowerCase().contains(search)) {
      return true;
    }
    if (homePhone.toLowerCase().contains(search)) {
      return true;
    }
    if (officePhone.toLowerCase().contains(search)) {
      return true;
    }
    if (email.toLowerCase().contains(search)) {
      return true;
    }
    if (alertType.toLowerCase().contains(search)) {
      return true;
    }
    if (alertPriority.toLowerCase().contains(search)) {
      return true;
    }
    if (dateCreated.toString().toLowerCase().contains(search)) {
      return true;
    }
    if (loanNumber.toString().toLowerCase().contains(search)) {
      return true;
    }
    return false;
  }
}
