part of unify;

class MarketingCampaign
    implements UnifyObject, QueryObject, SortableObject<MarketingCampaign> {
  MarketingCampaign(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get id => json["Contact_Campaign_ID"];

  String get contactId => json["Contact_ID"];

  String get leadId => json["Lead_ID"];

  String get name => json["Campaign_Name"];

  String get fullName => "$firstName $lastName".trim();

  String get displayName => "$lastName, $firstName";

  String get firstName => json["First_Name"];

  String get lastName => json["Last_Name"];

  String get status => json["Status"];

  String get campaignStatus => json["Campaign_Status"];

  bool get isLead => json["Is_Lead_Campaign"];

  DateTime get dateCreated =>
      DateTime.tryParse(json["Campaign_Creation_Date"] as String);

  DateTime get endDate =>
      DateTime.tryParse(json["Campaign_End_Date"] as String);

  String get type => json["Campaign_Type"];

  String get companyCategory => json["Company_Category_Name"];

  int get events => json["Campaign_Event_Count"];

  @override
  List<String> sortFields = [
    "campaign_name",
    "first_name",
    "last_name",
    "date_created",
    "end_date",
    "type",
    "status",
  ];

  @override
  int compareTo(
      MarketingCampaign object, String sortField, bool sortAscending) {
    int response = 0;
    MarketingCampaign objectA = sortAscending ? this : object;
    MarketingCampaign objectB = sortAscending ? object : this;

    switch (sortField) {
      case "campaign_name":
        response = objectA?.name?.compareTo(objectB.name) ?? 0;
        break;
      case "first_name":
        response =
            (objectA?.firstName ?? "")?.compareTo(objectB.firstName ?? "") ?? 0;
        break;
      case "last_name":
        response =
            (objectA?.lastName ?? "")?.compareTo(objectB.lastName ?? "") ?? 0;
        break;
      case "date_created":
        response = objectA?.dateCreated?.compareTo(objectB.dateCreated) ?? 0;
        break;
      case "end_date":
        response = objectA?.endDate?.compareTo(objectB.endDate) ?? 0;
        break;
      case "type":
        response = objectA?.type?.compareTo(objectB.type) ?? 0;
        break;
      case "status":
        response =
            objectA?.campaignStatus?.compareTo(objectB.campaignStatus) ?? 0;
        break;
    }

    if (response == 0) {
      return objectA.name.compareTo(objectB.name);
    } else {
      return response;
    }
  }

  @override
  bool matchesSearch(String search) {
    if (search == null || search.isEmpty) {
      return true;
    }

    search = search.toLowerCase();

    if (name?.toLowerCase()?.contains(search) ?? false) {
      return true;
    }
    if (firstName?.toLowerCase()?.contains(search) ?? false) {
      return true;
    }
    if (lastName?.toLowerCase()?.contains(search) ?? false) {
      return true;
    }
    if (status?.toLowerCase()?.contains(search) ?? false) {
      return true;
    }
    if (type?.toLowerCase()?.contains(search) ?? false) {
      return true;
    }
    if (dateCreated?.toString()?.toLowerCase()?.contains(search) ?? false) {
      return true;
    }
    if (endDate?.toString()?.toLowerCase()?.contains(search) ?? false) {
      return true;
    }
    return false;
  }
}
