part of unify;

class VideoDetails implements UnifyObject {
  VideoDetails(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get id => json["id"];

  String get name => json["name"];

  String get description => json["description"];

  String get thumbnail => json["thumbnailUrl"];

  String get video => json["videoUrl"];

  String get download => json["downloadUrl"];

  num get width => json["width"];

  num get height => json["height"];

  Duration get duration => Duration(seconds: json["duration"]);

  num get views => num.tryParse(json["views"].toString());

  num get plays => num.tryParse(json["plays"].toString());

  String getThumbnail({int quality = 100, int width = 300}) =>
      json["thumbnailUrl"] + "/quality/$quality/width/$width/";

  DateTime get dateCreated =>
      DateTime.fromMillisecondsSinceEpoch(json["createdAt"]);

  DateTime get dateModified =>
      DateTime.fromMillisecondsSinceEpoch(json["updatedAt"]);

  String get displayName => "$name";

  // -- Actions --
  Future<String> share({String template = "MESSAGE"}) async {
    return client.getVideoShareUrl(id, template: template);
  }

  Future<String> preview(
      {String template = "MESSAGE", String name, String body}) async {
    return client.getVideoPreview(id,
        template: template, name: name, body: body);
  }

  Future edit({String name, String description}) {
    return client.editVideoInfo(id, name: name, description: description);
  }
}
